/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWBASICRENDERER_H
#define MAFWBASICRENDERER_H

#include <MafwRenderer.h>
#include <MafwRendererPolicy.h>

#include <QObject>
#include <QUrl>

class MafwRendererPlaylistHandler;
class MafwRendererCommandEventTransition;
class MafwRendererPolicyCommunicator;
class QStateMachine;
class QAbstractState;
class QState;

/**
 * This class provides some basic renderer functionality. Inherit from this
 * class to easily create a renderer. You'll only need to implement the few
 * pure virtual methods to make this a functional MafwRenderer.
 *
 * @credential TrackerReadAccess  In case the application uses playlists containing local content
 * @credential TrackerWriteAccess In case the application uses playlists containing local content
 */
class MafwBasicRenderer : public MafwRenderer
{
    Q_OBJECT

    struct MafwBasicRendererCallbackInfo {
        //! Object waiting for reply
        QObject*           m_receiver;

        //! Callback slot
        const char*        m_member;

        //! Object receiving error message
         QObject*          m_errorReceiver;

         //! Error callback slot
         const char*       m_errorMember;
    };

protected:
    /**
     * Default timeout for waiting response from all do* functions
     */
    static const uint MAFW_RENDERER_PENDING_DELAY;

    /**
     * Default timeout for waiting response from policy
     */
    static const uint MAFW_POLICY_PENDING_DELAY;

public:

    /**
     * Constructor
     * @param uuid Mafw uuid for the renderer.
     * @param pluginName The name of plugin which created the renderer
     * @param name The human readable name tof the renderer
     * @param parent The parent of this object, if any
     */
    MafwBasicRenderer(const QString& uuid,
                      const QString& pluginName,
                      const QString& name,
                      QObject *parent = 0);

    /**
     * Desctructor
     */
    ~MafwBasicRenderer();

    /**
     * Initializes the instance
     * @return True if successful, false otherwise.
     */
    bool initialize();

    /**
     * Enumeration for currently playing item
     */
    enum MafwRendererPlayingUri {
        UnknownUri,
        CurrentUri,
        NextUri
    };

    /**
     * Loads and applies renderer policy plugin with then given name.
     * @param name The name of the renderer policy plugin
     * @param group The group this renderer belongs to.
     * @return true if the policy could be set, false otherwise
     * @note You may set needed resources with MafwRendererPolicy::setDefaultResources.
     * @see MafwRendererPolicy::PolicyGroup
     */
    bool setRendererPolicy(const QString& name, MafwRendererPolicy::PolicyGroup group);

    /**
     * Sets the renderer policy object to given policy. The ownership of the
     * policy object is transferred.
     * @param policy The policy object to use on the renderer.
     * @param group The group this renderer belongs to.
     * @return true if the policy could be set, false otherwise
     * @note You may set needed resources with MafwRendererPolicy::setDefaultResources.
     * @see MafwRendererPolicy::PolicyGroup
     */
    bool setRendererPolicy(MafwRendererPolicy *policy, MafwRendererPolicy::PolicyGroup group);

    /**
     * Sets the renderer policy to default policy if such exists. Sets default resource set to Audio and Video.
     * @param group The group this renderer belongs to.
     * @return true if the default policy could be set, false otherwise
     * @note You may alter needed resources with MafwRendererPolicy::setDefaultResources.
     * @see MafwRendererPolicy::PolicyGroup
     * @see MafwRendererPolicy::PolicyResourceFlag
     */
    bool setDefaultRendererPolicy( MafwRendererPolicy::PolicyGroup group );
    /**
     * @deprecated Use the one with MafwRendererPolicy::PolicyGrou parameter
     */
    bool setDefaultRendererPolicy();

    /**
      * Gets current policy object.
      * @return Policy
      */
    MafwRendererPolicy* rendererPolicy() const;

public: //MafwRenderer interface implementation
    bool play();
    bool play(const QUrl &url, uint startPosition = 0);
    bool play(const QUrl &url, const QString &contentType);

    bool stop();

    bool pause();
    bool pauseAt(uint position);
    bool resume();

    bool getStatus(QObject* resultsReceiver, const char* resultsMember);

    bool assignPlaylist(MafwPlaylist* playlist);

    bool next();
    bool previous();


    bool gotoIndex(unsigned int index);

    bool setPosition(int seconds, SeekMode mode = SeekAbsolute);
    bool getPosition(QObject* resultsReceiver,
                     const char* resultsMember);

    bool setMafwProperty(const QString &name, const QVariant &value);


public: //MafwBasicRenderer abstract part
    /**
     * Implement this to start playing the given uri.
     * Implementation must emit rendererPlaying-signal when playback has started.
     * @param content to play.
     * @deprecated Use the one with MafwMediaInfo parameter
     */
    virtual void doPlay(const MafwContent& content) = 0;

    /**
     * Implement this to start playing the given media info.
     * Implementation must emit rendererPlaying-signal when playback has started.
     *
     * @param mediaInfo The media to play.
     * @note Some interesting info is provided via the mediaInfo-parameter.
     * Always:
     * - The URL, [MAFW_METADATA_KEY_URI], The URI to play.
     * Optionally:
     * - The usage count, [MAFW_METADATA_KEY_PLAY_COUNT] How many times the
     * media has been played.
     * - The pause position, [MAFW_METADATA_KEY_PAUSED_POSITION] The position
     * where the media should be immediately paused when starting to play.
     *
     * @see MafwBasicRenderer::setFetchProperties() if this function has been
     * called the metadata key is the iri and metadata value the corresponding iri's
     * value.
     */
    virtual void doPlay(const MafwMediaInfo& mediaInfo);

    /**
     * Implement this this to stop the current playback.
     * Implementation must emit rendererStopped-signal when playback has stopped.
     */
    virtual void doStop() = 0;

    /**
     * Implement this to pause the current playback.
     * Implementation must emit rendererPaused-signal when playback has paused.
     */
    virtual void doPause() = 0;

    /**
     * Implement this to resume the paused playback.
     * Implementation must emit rendererResumed-signal when playback has resumed.
     */
    virtual void doResume() = 0;

    /**
     * Implement this to seek the current playback.
     * Implementation must emit rendererSeeking-signal when seeking has started. Emit rendererPlaying
     * when seeking has stopped.
     * @param position The position to jump in the playin item.
     * @param seeKMode Is seek relative or absolute seek
     */
    virtual void doSeek(int position, MafwRenderer::SeekMode seekMode) = 0;

    /**
     * Implement this to automatically to go to next playable item on assigned
     * playlist when the current item ends. Automatically means that renderer 
     * should go to the next item without going through stop state.
     * @param content to play next
     * @return True if the hint will be used
     * @deprecated Use the one with MafwMediaInfo parameter
     */
    virtual bool doNextHint(const MafwContent& next) = 0;

    /**
     * Implement this to automatically to go to next playable item on assigned
     * playlist when the current item ends. Automatically means that renderer
     * should go to the next item without going through stop state.
     * @note When the playlist content or its repeat mode changes this function may be called
     * with an empty MafwMediaInfo object to signify that previously given valid
     * mediainfo should not be played, i.e. playback should stop after currently
     * playing mediainfo is finished.
     *
     * @param next The mediaInfo to play next.
     * @return True if the hint will be used.
     */
    virtual bool doNextHint(const MafwMediaInfo& next);

Q_SIGNALS:

    /**
     * Emit when renderer has stopped.
     */
    void rendererStopped();

    /**
     * Emit when renderer starts playback
     * @param item "Current" if the playing item is one given in doPlayURI,
     * "Next" if the playing item is one given in doNextHint(). Unknown
     * if the renderer is playing some third party content.
     */
    void rendererPlaying(int item);

    /**
     * Emit when renderer has paused.
     */
    void rendererPaused();

    /**
     * Emit when renderer has resumed
     */
    void rendererResumed();

    /**
     * Emit when renderer starts seeking
     */
    void rendererSeeking();

    /**
     * Emit when renderer has reached end-of-stream
     */
    void rendererEos();

    /**
     * Emit when error is encountered.
     */
    void rendererError(const MafwError& error);

    /**
     * Signals that renderer is ready to receive/cache the next item to be played.
     * This is currently implied by the rendererPlaying() signal also, but this can be
     * used without state transitions and in special cases e.g. when immediate pause is requested via pauseAt().
     * @param item MafwBasicRenderer::MafwRendererPlayingUri enum of the current item
     */
    void rendererReadyForNext(MafwBasicRenderer::MafwRendererPlayingUri item);

private Q_SLOTS:
    /**
     * Keeps track of the state, if someone requests it
     */
    void updateState(MafwRenderer::State currentState);

    /**
     * Unassign the given playlist if it is assigned.
     * Used when a playlist has been deleted, so don't trust the playlist pointer
     * to be valid anymore.
     * @param playlist The playlist to unassign, can be invalid pointer.
     */
    void unAssignPlaylist(const MafwPlaylist* playlist);

    /**
     * Sends status information to requesters
     */
    void sendStatusInfo();

    /**
     * Tracks policy signals and emits state machine events based on them
     * @param event Policy event type
     */
    void trackPolicy(MafwRendererPolicy::PolicyEvent policyEvent);

    /**
     * Converts Playlisthandler's item updates to MafwContent format.
     * @param index The index in the playlist
     * @param mediaInfo The media info object of the playing item
     */
    void emitOldMediaChanged(int index, const MafwMediaInfo &mediaInfo);

    /**
     * Tracks rendererEos signals
     */
    void eos();

    /**
     * Reacts to renderer's ready for next item signals
     */
    void rendererImplementationReadyForNext(MafwBasicRenderer::MafwRendererPlayingUri item);

private:
    /**
     * Initializes the state machine states and transitions.
     * @return True on success, false otherwise
     */
    bool initializeStateMachine();

    /**
     * Configures the Play URI states.
     * Play URI is separate state so it won't confuse the playlist playing states
     * @param parent The state containing the Play URI functionality and states.
     */
    void configurePlayURIStates(QState* parent);

private:

    /** The Qt state machine instance */
    QStateMachine *m_stateMachine;
    /** Playlist handler for fecthing playable item's metadata etc */
    MafwRendererPlaylistHandler *m_playlistHandler;
    /** Current state of the renderer */
    MafwRenderer::State m_state;
    /** List of getStatus transaction requests */
    QList<MafwBasicRendererCallbackInfo*> m_statusTransactions;
    /** Renderer policy object */
    MafwRendererPolicy *m_policy;
    /** Mediator between the statemachine and renderer policy events */
    MafwRendererPolicyCommunicator *m_policyCommunicator;
};

#endif // MAFWBASICRENDERER_H
