/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWCALLBACKHELPER_H_
#define MAFWCALLBACKHELPER_H_

#include <QtGlobal>

class MafwRenderer;
class MafwMediaInfo;
class MafwError;
class QObject;
class QMetaMethod;
class QString;

/**
 * @brief This is a helper class for invoking callback methods and
 * checking their validity.
 */

class MafwCallbackHelper
{
public:

    /**
     * Returns QMetaMethod for given object and signature.
     * @param targetObject Object in question
     * @param slotSignature Signature
     * @param methodOut QMetaMethod will be returned to this
     * @return true If callback method was found
     */
    static bool getCallbackMethod(const QObject* targetObject,
                                  const char* slotSignature,
                                  QMetaMethod& methodOut);

    /**
     * Invokes the error callback slot having signature like
     * "void example_slot(const MafwError& error)" or
     * "void example_slot(const MafwError& error, MafwRenderer& rndr)"
     *
     * @param renderer Pointer to associated MafwRenderer.
     * @param error The error being delivered to the callback slot.
     * @param object The object whose callback is invoked.
     * @param slot The slot in callback object which is invoked.
     */
    static void invokeErrorCallback(MafwRenderer* renderer,
                                    MafwError& error,
                                    QObject* object,
                                    const char* slot);

    /**
     * Checks that the error callback has correct signature:
     * "void example_slot(const MafwError& error)" or
     * "void example_slot(const MafwError& error, MafwRenderer& rndr)"
     *
     * @param object The QObject owning the slot
     * @param slot The slot to be checked.
     * @return true if signature is of correct type, false if not.
     */
    static bool checkErrorCallback(QObject* object, const char* slot);

    /**
     * Invokes the status callback slot.
     *
     * @param renderer Pointer to associated MafwRenderer.
     * @param playlistId The ID of the current playlist
     * @param playbackIndex The index of the currently playing playlist item.
     * @param playbackState The current state of the renderer.
     * @param objectId The object ID of the current item.
     * @param object The object whose callback is invoked.
     * @param slot The slot in callback object which is invoked.
     */
    static void invokeGetStatusCallback(MafwRenderer* renderer,
                                        const QString& playlistId,
                                        uint playbackIndex,
                                        int playbackState,
                                        const QString& objectId,
                                        QObject* object,
                                        const char* slot);

    /**
     * Checks the validity of the callback slot for GetStatus.
     *
     * @param object The QObject owning the slot
     * @param slot The slot which is checked.
     * @return true if signature is of correct type, false if not.
     */
    static bool checkGetStatusCallback(QObject* object, const char* slot);

    /**
     * Invokes the position callback slot.
     *
     * @param renderer Pointer to associated MafwRenderer.
     * @param position The position being delivered to the callback slot.
     * @param object The object whose callback is invoked.
     * @param slot The slot in callback object which is invoked.
     */
    static void invokeGetPositionCallback(MafwRenderer* renderer, uint position,
                                          QObject* object, const char* slot);

    /**
     * Checks the validity of position callback slot.
     *
     * @param object The QObject owning the slot
     * @param slot The slot in callback object which is connected.
     * @return true if signature is of correct type, false if not.
     */
    static bool checkGetPositionCallback(QObject* object, const char* slot);

    /**
     * Checks the validity of callback slot for getCurrentMediaInfo().
     *
     * @param object The QObject owning the slot.
     * @param slot The slot in callback object which is connected.
     * @return true if signature is of correct type, false if not.
     */
    static bool checkGetMediaInfoCallback(QObject* object, const char* slot);

    /**
     * Invokes the get current media info callback slot.
     *
     * @param renderer Pointer to associated MafwRenderer.
     * @param mediaInfo The MafwMediaInfo object to be passed in the callback.
     * @param object The object whose callback is invoked.
     * @param slot The slot in callback object which is invoked.
     */
    static void invokeGetMediaInfoCallback(MafwRenderer* renderer,
											const MafwMediaInfo& mediaInfo,
											QObject* object, const char* slot);

private:
    /**
     * Cheks the connectivity of given signal and slot
     *
     * @param signature The expected signature.
     * @param slot The receiver slot.
     * @return true, if the signature is of expected type, false otherwise.
     */
    static bool checkCallbackSignature(const char* signature, const char* slot);
};

#endif /*MAFWCALLBACKHELPER_H_*/
