/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_CONTENT_H
#define MAFW_CONTENT_H

#include <QVariant>
#include <QString>
#include <QHash>

class MafwSource;

/**
 * @brief Presents the container for metadata.
 *
 * This class is used to store metadata of media entities such as music or 
 * video clips or images. 
 * 
 * @note This class can contain just one value for one key, 
 * so it is recommended to use MafwMediaInfo instead of this class.
 */
class MafwContent
{
public:
        /**
         * Default constructor
         */
        MafwContent();

	/** 
	 * Construct a new content object for the given uuid. 
	 * @param uuid universal unique identifier of content to be created
	 */
	MafwContent( const QString& uuid );

	/**
	 * Constructs a copy of other. This operation takes constant time, 
	 * because MafwContent is implicitly shared. This makes returning a 
	 * MafwContent from a function very fast. If a shared instance is 
	 * modified, it will be copied (copy-on-write), and that takes linear time.
     * @see operator=(). 
     * @param other MafwContent to be copied
	 */
	MafwContent( const MafwContent& other );

	/** Destructor */
	~MafwContent();

	/** 
	 * Assigns other to this MafwContent and returns a reference to this 
	 * MafwContent. 
	 * @param other MafwContent to be assigned
	 */
	MafwContent& operator=( const MafwContent& other );

public:
	/** 
	 * Get this object's universally-unique identifier. 
	 * @return universally-unique identifier of this content
	 */
	QString uuid() const;

	/**
	 * Get a constant map reference to this item's metadata. This can be 
	 * used directly in aQMapIterator to iterate thru the data.
	 * @return item's metadata
	 */
	const QMap <QString,QVariant> metaData() const;

	/**
	 * Get value for given metadata key.
	 * @param metaDataKey Metadata key which value is of interest.
	 * @return metadata for given key as QVariant. 
	 * If given metadata key was not found, invalid QVariant is returned.
	 */
	QVariant metaData(QString metaDataKey) const;
	
	/**
	 * Set new/updated metadata keys and their values to the content object. 
	 * This should be used only when getting content for an action
	 * @note This function is not designed to be used from UI.
	 * @todo Evaluate if this function should be only in mafw-shared project. 
	 */
	void setMetaData(QMap <QString,QVariant>& metaData,
			 const QObject* receiver = NULL,
			 const char* slot = NULL);

private:
    /** universally-unique identifier of this content */
	QString m_uuid;
	
	/** item's metadata */
	QMap <QString,QVariant> m_metaData;
};

#endif
