/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_ERROR_H
#define MAFW_ERROR_H

#include <QString>
#include <QMetaType>

/**
 * @brief The class representing the error with code and message information.
 *
 * This class is used to inform some error in MafwSource or MafwRenderer,
 * for example MafwRenderer::error().
 */
class MafwError
{
public:
    /** @enum Code Enumeration containing possible error codes. */
    enum Code
    {
        NoError = 0,/**< No error */
        Uninitialised,/**< Uninitialized error */
        NotImplementedError,/**< Not implemented error */
        NothingButErrors,/**< General error */

        /* Callback related errors */
        CallbackGeneral = 40,/**< General callback error */
        CallbackSlotNotFound,/**< Result slot not found error */
        CallbackErrorSlotNotFound,/**< Error slot not found error */
        CallbackCouldNotInvoke,/**< Result (or error) slot couldn't be invoked error */
        CallbackInvalidArguments,/**< Result (or error) slot has invalid arguments */

        /* Source errors,  codes have 1:1 mapping with FMAFW */
        SourceError_InvalidObjectId = 400,/**< Invalid source object id */
        SourceError_ObjectIdNotAvailable,/**< Object id of the source not available */
        SourceError_InvalidSearchString,/**< Invalid source search string */
        SourceError_InvalidSortString,/**< Invalid source sort string */
        SourceError_InvalidBrowseId,  /**< Invalid source browse id *//* Or bundle to invalid_params? */
        SourceError_Peer,/**< Source peer error */
        SourceError_BrowseResultFailed,/**< Failed source browse result */
        SourceError_GetMetadataResultFailed,/**< Failed source get metadata result */
        SourceError_UnsupportedMetadataKey,/**< Unsupported source metadata key */
        SourceError_DestroyObjectFailed,/**< Destroying source object failed */
        SourceError_PlaylistParsingFailed,/**< Source, parsing playlist failed */

        /* Renderer errors,  codes have 1:1 mapping with FMAFW */
        RendererError_NoMedia = 300,/**< No media for playback */
        RendererError_URINotAvailable,/**< URI not available */
        RendererError_InvalidURI,/**< Invalid URI */
        RendererError_MediaNotFound,/**< Media not found */
        RendererError_StreamDisconnected,/**< Stream disconnected */
        RendererError_TypeNotAvailable,/**< Type not available */
        RendererError_Playback,/**< Playback error */
        RendererError_UnableToPerform,/**< Unable to perform operation */
        RendererError_UnsuppertedType,/**< Unsupported type */
        RendererError_UnsupportedResolution,/**< Unsupported resolution */
        RendererError_UnsupportedFPS,/**< Unsupported FPS */
        RendererError_DRMNoLicense,/**< No DRM license */
        RendererError_DRMNotAllowed,/**< DRM playback not allowed */
        RendererError_DRMOther,/**< Other DRM error */
        RendererError_DeviceUnavailable,/**< Device not available */
        RendererError_CorruptedFile,/**< File corrupted */
        RendererError_PlaylistParsing,/**< Renderer, parsing playlist failed */
        RendererError_CodecNotFound,/**< Codec not found */
        RendererError_VideoCodeNotFound,/**< Video codec not found */
        RendererError_AudioCodecNotFound,/**< Audio codec not found */
        RendererError_NoPlaylist,/**< No playlist assigned, or empty playlist file in play uri case */
        RendererError_IndexOutOfBounds,/**< Index out of bounds */
        RendererError_CannotPlay,/**< Cannot play */
        RendererError_CannotStop,/**< Cannot stop */
        RendererError_CannotPause,/**< Cannot pause */
        RendererError_CannotSetPosition,/**< Cannot set position */
        RendererError_CannotGetPosition,/**< Cannot get position */
        RendererError_CannotGetStatus,/**< Cannot get status */
        RendererError_CannotSetProperty,/**< Cannot set property */
        RendererError_Denied,/**< Operation denied by policy */
        RendererError_DRMClockNotSet, /**< Invalid time, DRM no like*/
        RendererError_MmcNotAvailable
    };

    /**
      * <HR>
      * <b>For backwards compatibility with old gst-renderer.</b>
      * <HR>
      */
    static const QString MAFW_SOURCE_ERROR_QUARK;
    //="com.nokia.mafw.error.source";
    
    /** 
     * Construct a new error object with empty error message and code NoError. 
     */
    MafwError();
    
    /** 
     * Construct a new error object with given code and message. 
     * @param code The code of error to be created
     * @param message The human readable error message
     */
    MafwError(Code code, const QString& message);
    
    /**
     * Constructs a copy of other. This operation takes constant time, 
     * because MafwError is implicitly shared. This makes returning a 
     * MafwError from a function very fast. If a shared instance is modified, 
     * it will be copied (copy-on-write), and that takes linear time.
     * @see operator=(). 
     * @param other MafwError to be copied
     */
    MafwError(const MafwError& other);
    
    /** Destructor */
    ~MafwError();
    
    /** 
     * Assigns other to this MafwError and returns a reference to this 
     * MafwError. 
     * @param other MafwError to be assigned
     */
    MafwError& operator=(const MafwError& other);
    
    /**
     * Set the code of this error object
     * @param code The code to be set
     */
    void setCode(MafwError::Code code);
    
    /**
     * Get the code of this error object
     * @return the error code 
     */
    MafwError::Code code() const { return m_code; }

    /**
     * Set the human readable message for this error object.
     * @param message The message to be set
     */
    void setMessage(const QString& message);
    
    /**
     * Get the human readable message of this error object.
     * @return the message
     */
    QString message() const	{ return m_message; }
    
    /**
     * Get the string presentation of this error object
     * @return the string presentation of the error 
     */
    QString toString() const;
    
    /**
     * Get the string presentation of this error's code
     * @return the string presentation of code
     */
    static QString codeToString(MafwError::Code code);
    
protected:
    /** the code of error */
    Code m_code;
    
    /** the message of error */
    QString m_message;
};

Q_DECLARE_METATYPE(MafwError)

#endif
