/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWGSPLAYLIST_H_
#define MAFWGSPLAYLIST_H_

#include <QObject>

#include "MafwPlaylistBase.h"

/**
 * @brief The class providing implementation for playlist in MAFW.
 *
 * @code
 * //Compose "all songs"-query via qsparql.
 *  QSparqlQuery allSongsQuery (QString("SELECT ?_songUri "
 *      "<http://www.tracker-project.org/ontologies/tracker#id>(?_songUri) "
 *      "WHERE"
 *      "{"
 *          "?_songUri <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> <http://www.tracker-project.org/temp/nmm#MusicPiece> ."
 *      "}"));
 *  m_songsModel = new QSparqlQueryModel(MafwRegistry::instance());
 *  m_songsModel->setQuery(allSongsQuery, *m_connection);
 *
 *  //Get new unique playlist id. Unique playlist id is required for
 *  //publishing playlist to other processes.
 *  //(Usually also the renderer used to play playlist lays in other process.)
 *  QString playlistId = MafwPlaylist::getNewPlaylistId();
 *  //Create new playlist using unique playlist ID and (optional) playlist name.
 *  MafwGeneralSourcePlaylist* pl = new MafwGeneralSourcePlaylist(playlistId, "The sample playlist", MAFW_TRACKER_SOURCE_UUID);
 *  //Set playlist model
 *  pl->setUnderlyingModel(m_songsModel);
 *  //Publish playlist so that other processes (renderer) can use it
 *  bool published = MafwShared::instance()->publishPlaylist(pl);
 * @endcode
 *
 * There is one requirement for the model columns to make the playlist to work properly:
 * The first column contains the unique ID of the source's object. For example "urn:uuid:exampleuuid".
 * The above example query fulfills this requirement.
 *
 * @see MafwShared::publishPlaylist
 */
const QString MAFW_TRACKER_SOURCE_UUID = "MafwTrackerSource";

class MafwGeneralSourcePlaylistPrivate;

class MafwGeneralSourcePlaylist : public MafwPlaylistBase
{
    Q_OBJECT

public:

    /**
     * Constructor.
     * @param id the unique id of the playlist
     * @param name The name of the playlist.
     * @param sourceId id of source
     * @see getNewPlaylistId()
     */
    MafwGeneralSourcePlaylist( const QString& id, const QString& name = QString(), const QString& sourceId = MAFW_TRACKER_SOURCE_UUID);

    /**
     * The destructor
     */
    virtual ~MafwGeneralSourcePlaylist();

    // from MafwPlaylist
    MafwMediaInfo getItemMediaInfo(int index);
private:
    MafwGeneralSourcePlaylistPrivate* d;

};

#endif /*MAFWGSPLAYLIST_H_*/
