/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWINTERNALREGISTRY_H_
#define MAFWINTERNALREGISTRY_H_

#include <QMap>

#include "MafwRegistry.h"

class QPluginLoader;

/**
 * @brief The internal MAFW registry class.
 * 
 * MafwRegistry interface has the functions that are intended to be used by 
 * applications. The MafwInternalRegistry provides functions which inteded 
 * only to be used by plugins and internally in MAFW. 
 *
 * MafwPlugins can use this class for registering sources and renderers to 
 * registry.  
 * 
 * @see MafwRegistry
 */


class MafwInternalRegistry : public MafwRegistry
{
    Q_OBJECT

    /************************************************************************
     * Initialization
     ************************************************************************/
private:
    Q_DISABLE_COPY(MafwInternalRegistry)

protected:
    /**
     * The constructor.
     *
     * Only MafwInternalRegistry::internalInstance() or MafwRegistry::instance() 
     * can create a new registry instance.
     * 
     * @param parent the parent object
     */
    MafwInternalRegistry(QObject* parent);

public:
    /**
     * The destructor.
     */
    virtual ~MafwInternalRegistry();

    /**
     * The singleton constructor.
     * 
     * @note Never delete this singleton instance manually.
     * @return the MafwInternalRegistry singleton instance
     */
    static MafwInternalRegistry* internalInstance();

private:
    /** The singleton registry instance */
    static MafwInternalRegistry* m_instance;

public: //from MafwRegistry 
    MafwSource* source(const QString& uuid) const;
    const QList <MafwSource*> sources() const;
    MafwRenderer* renderer(const QString& uuid) const;
    const QList <MafwRenderer*> renderers() const;
    const QList <MafwPlaylist*> playlists() const;
    const QList <MafwPlaylist*> persistentPlaylists() const;    
    MafwPlaylist* playlist(const QString& id) const;
    void loadPlugins();
    void unloadPlugins();
    bool loadPlugin(const QString& name);
    void unloadPlugin(const QString& name);
    MafwPlugin* plugin(const QString& name) const;
    const QStringList loadedPlugins() const;
    const QStringList availablePlugins() const;    
    MafwPluginControlInterface* pluginControlInterface(const QString& pluginName) const;        
    const QList <MafwPluginControlInterface*> pluginControlInterfaces() const;    
    
    /************************************************************************
     * Sources
     ************************************************************************/
public:
    /** 
     * Add a source extension to this registry. Ownership and parenthood transferred to 
     * registry.
     * 
     * This function is for MafwPlugin and MAFW library internal use.
     * Source plugins should use this to register their sources to MAFW registry.
     * 
     * @code
     * bool QMafwExamplePlugin::addSourceToRegistry( QString name, QString pluginName, QString uuid )
     * {
     *     registry = MafwInternalRegistry::internalInstance();
     *     if( registry->source( uuid ) )
     *     {
     *         //source already exists
     *         return false;
     *     }
     *     QMafwExampleSource *source = new QMafwExampleSource( this, uuid, pluginName, name );
     *     return registry->addSource( source );
     * }
     * @endcode
     * @note MafwPlugin who adds source to registry are responsible also for removing it.
     * 
     * @return true if adding is successfull, false otherwise. Adding will fail always
     * if there already exists a source with same uuid or if given source is already 
     * registered.
     * @param source source to be added
     */
     bool addSource(MafwSource* source);

private:
    /** List of known sources. Owned. */
    QList <MafwSource*> m_sources;

    /************************************************************************
     * Renderers
     ************************************************************************/
public:
    /** 
     * Add a renderer extension to this registry. Ownership and parenthood are transferred to 
     * registry.
     * 
     * This function is for MafwPlugin and MAFW library internal use.
     * Renderer plugins should use this to register their renderers to MAFW registry.
     * 
     * @code
     * bool QMafwExamplePlugin::addRendererToRegistry( QString pluginName, QString uuid )
     * {
     *     registry = MafwInternalRegistry::internalInstance();
     *     if( registry->renderer( uuid ) )
     *     {
     *         //renderer already exists
     *         return false;
     *     }
     *     QMafwExampleRenderer *renderer = new QMafwExampleRenderer( this, uuid, pluginName );
     *     return registry->addRenderer( renderer );
     * }
     * @endcode
     * @note MafwPlugin who adds renderer to registry are responsible also for removing it.
     * 
     * @return true if adding is successfull, false otherwise. Adding will fail always
     * if there already exists a renderer with same uuid or if given renderer is already
     * registered. 
     * @param renderer renderer to be added
     */
    bool addRenderer(MafwRenderer* renderer);

private:
    /** List of known renderers. Owned. */
    QList <MafwRenderer*> m_renderers;

    /************************************************************************
     * Playlists
     ************************************************************************/
public:
    /* 
     * Add a renderer playlist to this registry. Ownership and parenthood are transferred to 
     * registry.
     * 
     * This function is for MAFW library internal use.
     * 
     * @return true if adding is successfull, false otherwise.
     * @param playlist playlist to be added
     */
    bool addPlaylist( MafwPlaylist* playlist );

    /*
     * Remove a playlist by its id. playlistRemoved() signal is emitted when 
     * removing has been done.
     * 
     * This function is for MAFW library internal use.
     *
     * @param id id of the playlist to be removed
     */
    void removePlaylist(const QString& id);
        
private:
    /** List of known playlists. Owned. */
    QList <MafwPlaylist*> m_playlists;
    
public:
    /**
     * Removes extension from registry and deletes it by its uuid. 
     * Extension can be either source or renderer.
     * 
     * Plugins should use this to unregister their sources from MAFW registry.
     * 
     * @param uuid uuid of the extension to be removed
     */
    void removeExtension( const QString& uuid );

    /************************************************************************
     * Plugins
     ************************************************************************/
public:
    /*
     * Adds given plugin control interface assosiated with given plugin name. 
     * Existing control interface with same plugin name is deleted.
     *
     * This function is for MAFW library internal use.
     *
     * @param pluginName the filename of plugin which control interface is 
     *        being added.
     * @param pluginControlIf the plugin control interface to be added. 
     *        Ownership transfers.
     */
    void addPluginControlInterface(const QString& pluginName, 
                                   MafwPluginControlInterface* pluginControlIf);
    
private:
    /** 
     * Mapping between plugin instance and their loaders is needed
     * to be able to unload plugins thru QPluginLoader::unLoad().
     * Owned.
     */
    QMap <MafwPlugin*,QPluginLoader*> m_pluginLoaders;
    
    /** 
     * Mapping between plugin names and their controllers.
     * Owned.
     */
    QMap <QString,MafwPluginControlInterface*> m_pluginControllers;

protected Q_SLOTS:
    /*
     * Registry connects this to extensions' destroyed() signals to be able
     * to remove them from its lists and emit *Removed() signals. 
     * 
     * This slot is for MAFW library internal use.
     * 
     * @param extension pointer to removed extension
     */
    void extensionDestroyed(QObject* extension);
    
};

#endif /*MAFWINTERNALREGISTRY_H_*/
