/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_MEDIA_INFO_H
#define MAFW_MEDIA_INFO_H


#include <QObject>
#include <QString>
#include <QVariant>

class MafwContent;

template <typename T1, typename T2>
class QMap;

/**
 * @brief Container class for metadata.
 *
 * MafwMediaInfo class acts as a container for metadata of media entities
 * such as music or video clips or images.
 *
 * MafwMediaInfo is used to simplify APIs that pass different metadata
 * key-value pairs.
 *
 * Please note that one metadata key may have several values, but usually only one.
 * For example, a music clip has only one title and one URI, but can have several performers.
 */
class MafwMediaInfo
{
public:

	/**
	 * The default constructor should not normally be used.
	 * The uuid (universally-unique identifier) of what  this object
	 * represents is left invalid (empty).
	 * Normally every MafwMediaInfo object should have an uuid
	 * to identify the medium (video clip, audio clip, image...) it's
	 * data represents.
	 *
	 * Using the constructor MafwMediaInfo( const QString& uuid ) is highly
	 * encouraged.
	 *
	 * @see setUuid(const QString&)
	 * @see MafwMediaInfo(const QString&)
	 */
	MafwMediaInfo();

	/**
	 * Construct a new content object for the given uuid.
	 * @param uuid Universal unique identifier of content to be created
	 */
	MafwMediaInfo( const QString& uuid );

	/**
	 * Constructs a copy of other. This operation takes constant time,
	 * because MafwMediaInfo is implicitly shared. This makes returning a
	 * MafwMediaInfo from a function very fast. If a shared instance is
	 * modified, it will be copied (copy-on-write), and that takes linear time.
     * @see operator=().
     * @param other MafwMediaInfo to be copied
	 */
	MafwMediaInfo( const MafwMediaInfo& other );

        /**
         * Backwards compatiblity copy-constructor. Will be deprecated hopefully
         * soon.
         * @note DO NOT USE! YOUR OWN RISK!
         */
        MafwMediaInfo( const MafwContent& mafwContent);

	/** Destructor */
	~MafwMediaInfo();

	/**
	 * Assigns other to this MafwMediaInfo and returns a reference to this
	 * MafwMediaInfo.
	 * @param other MafwMediaInfo to be assigned
	 */
	MafwMediaInfo& operator=( const MafwMediaInfo& other );
        MafwMediaInfo& operator=( const MafwContent& mafwContent );

public:
	/**
	 * Get this object's universally-unique identifier.
	 * @return universally-unique identifier of this content
	 */
	QString uuid() const;

	/**
	 * Set this object's universally-unique identifier.
	 * Normally every MafwMediaInfo object should have an uuid
	 * to identify the medium (video clip, audio clip, image...) it's
	 * data represents.
	 *
	 * @param uuid The new universally-unique identifier.
	 */
	void setUuid(const QString& uuid);

	/**
	 * Indicates if the medium this object represents can be played
	 * by a MafwRenderer. Practically checks if an URI string is empty.
	 *
	 * @return True if the data represented by this object can be played.
	 */
        bool isPlayable() const;

	/**
	 * Get a constant map reference to this item's metadata. This can be
	 * used directly in a QMapIterator to iterate thru the data.
	 * @return Item's metadata
	 */
	const QMap <QString,QList<QVariant> >& metaData() const;

	/**
	 * Get value for given metadata key.
	 *
	 * @param metaDataKey Metadata key which value is of interest.
         * @return Metadata for given key as list QVariants.
         * If given metadata key was not found, empty list is returned.
	 */
        QList<QVariant> metaData(const QString& metaDataKey) const;

        /**
         * Get the first value for given metadata key. The most common
         * scenario is that metaKey has only one metadata value. This
         * function is provided for convenience.
         *
         * @param metaDataKey Metadata key which value is of interest.
         * @return First metadata for given key as QVariant.
         * If given metadata key was not found, invalid QVariant is returned,
         * also if the given metadata key's list is empty.
         */
        QVariant firstMetaData(const QString &metaDataKey) const;

	/**
	 * Set new/updated metadata keys and their values to the content object.
	 * This function will replace any existing metadata this object may hold.
	 * @note This function is not designed to be used from UI.
	 * @note All old metadata will be removed. This means keys which do not exist
	 * in new metadata map will be lost.
	 * @param metaData New set of metaData to replace all previous key-valuelist pairs.
	 */
        void setMetaData(const QMap<QString, QList<QVariant> > &metaData);

        /**
         * Sets new/updated metadata keys and their values to the content object.
         * The metadata is converted to the type of QMap<QString, QList<QVariant>.
         * Provided for convenience
         * @param metadata The metadata to be set for this object.
         * Will be converted to the format that metadata() returns.
         *
         */
        void setMetaData(const QMap<QString, QVariant> &metadata );

	/**
	 * Appends new metadata key/valuelist pair to this object's metadata map.
	 * If values for this key exist already, they will be replaced.
	 *
	 * @param key The key for these values
	 * @param metaData Actual metadata values that correspond to key
	 */
	void appendMetaData(QString key, QList<QVariant> metaData);

	/**
	 * Removes a metadata key/valuelist pair from this object's metadata map.
	 *
	 * @param key The key for these values
	 * @return Number of keys removed
	 */	
	int removeMetaData(QString key);
	
	/**
	 * Indicates if this item is usable and not empty. 
	 * Checks if uuid string is empty.
	 * @return True if the item is valid.
	 */	
	bool isValid();
	
private:
        /**
         * Extracts metadata from QMap containing QVariant as value to MafwMediaInfo's
         * QList<QString> as value
         * @param metadata The signle value type metadata map.
         */
        void extractSingleValueMetadata(const QMap<QString, QVariant> &metadata);

private:
    /** universally-unique identifier of this info */
	QString m_uuid;

	/** item's metadata */
	QMap <QString,QList<QVariant> > m_metaData;
};

Q_DECLARE_METATYPE(MafwMediaInfo)

#endif //MAFW_MEDIA_INFO_H
