/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWMESSAGEHANDLER_H_
#define MAFWMESSAGEHANDLER_H_

#include <QtGlobal>
#include <QObject>

class QTextStream;

/**
 * @brief MafwMessageHandler provides MAFW Qt message handler. 
 * 
 * The message handler is a function that prints out debug messages, 
 * warnings, critical and fatal error messages. MafwMessageHandler works just
 * like the default message handler, but it suppresses debug level messages
 * if QMAFW_LOG environment variable has not been set. Another additional
 * feature is that Glib debugs is converted to Qt debugs.
 */
namespace MafwMessageHandler
{
    /**
     * @brief Initialize MAFW log handling for this process.
     *
     * Reads QMAFW_LOG environment variable and assigns a custom
     * message handler. All Qt and Glib debug printing is then routed
     * through this custom handler.
     * 
     * The purpose of the custom message handler is to minimize
     * output prints to stderr/console.
     * 
     * QMAFW_LOG=1 is debug level
     * QMAFW_LOG=2 is warning level
     * QMAFW_LOG=3 is critical level
     * Default is warning level.
     * 
     * Disabling the custom message handler returns Qt's default
     * message handling.
     * 
     * This function is intended to be called as soon as possible
     * during application startup, but is safe to call at any time.
     * 
     * @param enabled enables/disables the custom message handler
     * @param stream the stream to write. If 0, then debugs are
     * written to standard output (stderr). Ownership transfers, stream is 
     * freed by recalling initMafwLogging (eg. initMafwLogging(false)).
     */
    void initMafwLogging(bool enabled=true, QTextStream* stream=0);

    class MessageHandlerRestorer : public QObject
    {
        Q_OBJECT

        private slots:
            void restoreDefaultMsgHandler();
    };

};

#endif /*MAFWMESSAGEHANDLER_H_*/
