/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWMETADATA_H_
#define MAFWMETADATA_H_

/**
 * SECTION: mafwmetadata
 * @short_description: metadata handling.
 *
 * List of well-known metadata keys to be used in mafw_source_get_metadata(),
 * mafw_renderer_get_metadata() and mafw_source_browse().  It is not guaranteed
 * all sources understand all of the keys listed here, but if they do they
 * will recognize the referred metadata by these names.
 */

/*
 * This file can be extended later, if we support more complex
 * metadata values.
 */

/**
 * MAFW_METADATA_KEY_URI:
 *
 * The URI by which any renderer supporting the schema should be able to
 * locate the item.  The value of this metadata is a string, as
 * defined in RFC 3986.metadata
 * @note URI is in its percent-encoded form.
 */
#define MAFW_METADATA_KEY_URI           "uri"

/**
 * MAFW_METADATA_KEY_TYPE:
 *
 * Describes the type of the item.  Its value is a string.
 */
#define MAFW_METADATA_KEY_TYPE          "type"

/**
 * MAFW_METADATA_KEY_MIME:
 *
 * Describes the type and format of the item.  Its value is a string
 * of &lt;type&gt; '/' &lt;subtype&gt; defined in RFC2045 section 5.1.
 * The value should be interpreted according to that standard.
 */
#define MAFW_METADATA_KEY_MIME          "mime-type"

/**
 * MAFW_METADATA_VALUE_MIME_CONTAINER:
 *
 * This is the MIME type of a container provided by MAFW source components.
 */
#define MAFW_METADATA_VALUE_MIME_CONTAINER      "x-mafw/container"

/**
 * MAFW_METADATA_KEY_TITLE:
 *
 * Describes the title of the item. Its value is a string.
 */
#define MAFW_METADATA_KEY_TITLE     "title"

/**
 * MAFW_METADATA_KEY_DURATION:
 *
 * Describes the duration of the item (in seconds).  Its value is an integer.
 */
#define MAFW_METADATA_KEY_DURATION      "duration"

/**
 * MAFW_METADATA_KEY_ARTIST:
 *
 * Describes the artist of the item.  Its value is a string.
 */
#define MAFW_METADATA_KEY_ARTIST        "artist"

/**
 * MAFW_METADATA_KEY_ALBUM:
 *
 * Describes the album of the item.  Its value is a string.
 */
#define MAFW_METADATA_KEY_ALBUM     "album"

/**
 * MAFW_METADATA_KEY_ORGANIZATION:
 *
 * Describes the organization of the item.  Its value is a string.
 */
#define MAFW_METADATA_KEY_ORGANIZATION      "organization"

/**
 * MAFW_METADATA_KEY_GENRE:
 *
 * Describes the genre of the item.  Its value is a string.
 */
#define MAFW_METADATA_KEY_GENRE     "genre"

/**
 * MAFW_METADATA_KEY_TRACK:
 *
 * Describes the track of the item.  Its value is an integer.
 */
#define MAFW_METADATA_KEY_TRACK     "track"

/**
 * MAFW_METADATA_KEY_YEAR:
 *
 * Describes the year of the item.  Its value is an integer.
 */
#define MAFW_METADATA_KEY_YEAR          "year"

/**
 * MAFW_METADATA_KEY_BITRATE:
 *
 * Describes the bitrate of the item.  Its value is an integer.
 */
#define MAFW_METADATA_KEY_BITRATE       "bitrate"

/**
 * MAFW_METADATA_KEY_COUNT:
 *
 * Describes how may times the item exists. Its value is an integer.
 */
#define MAFW_METADATA_KEY_COUNT     "count"

/**
 * MAFW_METADATA_KEY_PLAY_COUNT:
 *
 * Describes how may times the item has been played/viewed. Its value is an
 * integer.
 */
#define MAFW_METADATA_KEY_PLAY_COUNT        "play-count"

/**
 * MAFW_METADATA_KEY_LAST_PLAYED:
 *
 * Describes the time in epoch, when the item was played last time. Its value
 * is a long integer.
 */
#define MAFW_METADATA_KEY_LAST_PLAYED       "last-played"

/**
 * MAFW_METADATA_KEY_DESCRIPTION:
 *
 * A human readable description of the item. Its value is string.
 */
#define MAFW_METADATA_KEY_DESCRIPTION       "description"

/**
 * MAFW_METADATA_KEY_ENCODING:
 *
 * Describes how the item is encoded. Its value is a string.
 */
#define MAFW_METADATA_KEY_ENCODING      "encoding"

/**
 * MAFW_METADATA_KEY_ADDED:
 *
 * Date when item was added into the database. Its value is a long
 * integer.
 */
#define MAFW_METADATA_KEY_ADDED     "added"

/**
 * MAFW_METADATA_KEY_THUMBNAIL_URI:
 *
 * URI pointing to a thumbnail. Its value is a string.
 */
#define MAFW_METADATA_KEY_THUMBNAIL_URI "thumbnail-uri"

/**
 * MAFW_METADATA_KEY_THUMBNAIL_SMALL_URI:
 *
 * URI pointing to a small thumbnail. Its value is a string.
 */
#define MAFW_METADATA_KEY_THUMBNAIL_SMALL_URI   "thumbnail-small-uri"

/**
 * MAFW_METADATA_KEY_THUMBNAIL_MEDIUM_URI:
 *
 * URI pointing to a medium thumbnail. Its value is a string.
 */
#define MAFW_METADATA_KEY_THUMBNAIL_MEDIUM_URI  "thumbnail-medium-uri"

/**
 * MAFW_METADATA_KEY_THUMBNAIL_LARGE_URI:
 *
 * URI pointing to a large thumbnail. Its value is a string.
 */
#define MAFW_METADATA_KEY_THUMBNAIL_LARGE_URI   "thumbnail-large-uri"

/**
 * MAFW_METADATA_KEY_PAUSED_THUMBNAIL_URI:
 *
 * URI pointing to the thumbnail of the frame where video was
 * paused. Its value is a string.
 */
#define MAFW_METADATA_KEY_PAUSED_THUMBNAIL_URI   "paused-thumbnail-uri"

/**
 * MAFW_METADATA_KEY_PAUSED_POSITION:
 *
 * Position where the item is paused. Its value is an integer.
 */
#define MAFW_METADATA_KEY_PAUSED_POSITION   "paused-position"

/**
 * MAFW_METADATA_KEY_START_POSITION:
 *
 * Position where the item will start playing. Its value is an unsigned integer.
 */
#define MAFW_METADATA_KEY_START_POSITION   "start-position"

/**
 * MAFW_METADATA_KEY_THUMBNAIL:
 *
 * Image thumbnail. Its value is a byte array.
 */
#define MAFW_METADATA_KEY_THUMBNAIL     "thumbnail"

/**
 * MAFW_METADATA_KEY_IS_SEEKABLE:
 *
 * Describes if item is seekable. Its value is a boolean.
 */
#define MAFW_METADATA_KEY_IS_SEEKABLE       "is-seekable"

/**
 * MAFW_METADATA_KEY_RES_X:
 *
 * Describes the horizontal resolution of the item. Its value is an integer.
 */
#define MAFW_METADATA_KEY_RES_X     "res-x"

/**
 * MAFW_METADATA_KEY_RES_Y:
 *
 * Describes the vertical resolution of the item. Its value is an integer.
 */
#define MAFW_METADATA_KEY_RES_Y     "res-y"

/**
 * MAFW_METADATA_KEY_COMMENT:
 *
 * Comment describing the item. Its value is a string.
 */
#define MAFW_METADATA_KEY_COMMENT       "comment"

/**
 * MAFW_METADATA_KEY_TAGS:
 *
 * List of tags describing the item. Its value is a list of strings.
 */
#define MAFW_METADATA_KEY_TAGS          "tags"

/**
 * MAFW_METADATA_KEY_DIDL:
 *
 * Item metadata in DIDL lite format. Its value is a string.
 */
#define MAFW_METADATA_KEY_DIDL          "didl"

/**
 * MAFW_METADATA_KEY_ARTIST_INFO_URI:
 *
 * An URI pointing to artist description. Its value is a string.
 */
#define MAFW_METADATA_KEY_ARTIST_INFO_URI   "artist-info-uri"

/**
 * MAFW_METADATA_KEY_ALBUM_INFO_URI:
 *
 * An URI pointing to album information. Its value is an integer.
 */
#define MAFW_METADATA_KEY_ALBUM_INFO_URI    "album-info-uri"

/**
 * MAFW_METADATA_KEY_LYRICS_URI:
 *
 * An URI pointing to lyrics related to item. Its value is a string.
 */
#define MAFW_METADATA_KEY_LYRICS_URI        "lyrics-uri"

/**
 * MAFW_METADATA_KEY_LYRICS:
 *
 * Item lyrics. Its value is a string.
 */
#define MAFW_METADATA_KEY_LYRICS        "lyrics"

/**
 * MAFW_METADATA_KEY_RATING:
 *
 * Describes the rating of the item. Its value is an integer.
 */
#define MAFW_METADATA_KEY_RATING        "rating"

/**
 * MAFW_METADATA_KEY_COMPOSER:
 *
 * Describes the composer of the item. Its value is a string.
 */
#define MAFW_METADATA_KEY_COMPOSER      "composer"

/**
 * MAFW_METADATA_KEY_FILENAME:
 *
 * Original filename of the item resource. Its value is a string.
 */
#define MAFW_METADATA_KEY_FILENAME      "filename"

/**
 * MAFW_METADATA_KEY_FILESIZE:
 *
 * Size (in bytes) of the resource file pointed by the item. Its value is
 * an integer.
 */
#define MAFW_METADATA_KEY_FILESIZE      "filesize"

/**
 * MAFW_METADATA_KEY_COPYRIGHT:
 *
 * Copyright disclaimer. Its value is a string.
 */
#define MAFW_METADATA_KEY_COPYRIGHT     "copyright"

/**
 * MAFW_METADATA_KEY_PROTOCOL_INFO:
 *
 * Describes how may times the item exists. Its value is a string.
 * Specified in UPnP ConnectionManager documentation. 
 */
#define MAFW_METADATA_KEY_PROTOCOL_INFO "protocol-info"

/**
 * MAFW_METADATA_KEY_AUDIO_BITRATE:
 *
 * The bitrate used to encode the item. Its value is an integer.
 */
#define MAFW_METADATA_KEY_AUDIO_BITRATE "audio-bitrate"

/**
 * MAFW_METADATA_KEY_AUDIO_CODEC:
 *
 * The coder/decoder information for item. Its value is a string.
 */
#define MAFW_METADATA_KEY_AUDIO_CODEC       "audio-codec"

/**
 * MAFW_METADATA_KEY_ALBUM_ART_URI:
 *
 * An URI pointing to the original album art image. Its value is a string.
 */
#define MAFW_METADATA_KEY_ALBUM_ART_URI "album-art-uri"

/**
 * MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI:
 *
 * An URI pointing to small album art image. Its value is a string.
 */
#define MAFW_METADATA_KEY_ALBUM_ART_SMALL_URI   "album-art-small-uri"

/**
 * MAFW_METADATA_KEY_ALBUM_ART_MEDIUM_URI:
 *
 * An URI pointing to medium album art image. Its value is a string.
 */
#define MAFW_METADATA_KEY_ALBUM_ART_MEDIUM_URI  "album-art-medium-uri"

/**
 * MAFW_METADATA_KEY_ALBUM_ART_LARGE_URI:
 *
 * An URI pointing to large album art image. Its value is a string.
 */
#define MAFW_METADATA_KEY_ALBUM_ART_LARGE_URI   "album-art-large-uri"

/**
 * MAFW_METADATA_KEY_ALBUM_ART:
 *
 * Album art image. Its value is a byte array.
 */
#define MAFW_METADATA_KEY_ALBUM_ART     "album-art"

/**
 * MAFW_METADATA_KEY_RENDERER_ART_URI:
 *
 * Album art image uri coming from renderer. Its value is a string.
 */
#define MAFW_METADATA_KEY_RENDERER_ART_URI  "renderer-art-uri"

/**
 * MAFW_METADATA_KEY_VIDEO_BITRATE:
 *
 * The bitrate used to encode the item. Its value is an integer.
 */
#define MAFW_METADATA_KEY_VIDEO_BITRATE "video-bitrate"

/**
 * MAFW_METADATA_KEY_VIDEO_CODEC:
 *
 * The coder/decoder information for item. Its value is a string.
 */
#define MAFW_METADATA_KEY_VIDEO_CODEC       "video-codec"

/**
 * MAFW_METADATA_KEY_VIDEO_FRAMERATE:
 *
 * The amount of frames per second. Its value is a float.
 */
#define MAFW_METADATA_KEY_VIDEO_FRAMERATE   "video-framerate"

/**
 * MAFW_METADATA_KEY_VIDEO_SOURCE:
 *
 * Information about the source of the video. Provides information
 * like, for example, the hardware used to record it. Its value is a
 * string.
 */
#define MAFW_METADATA_KEY_VIDEO_SOURCE         "video-source"

/**
 * MAFW_METADATA_KEY_BPP:
 *
 * Describes how many bits are used to represent a pixel. Its value is an
 * integer.
 */
#define MAFW_METADATA_KEY_BPP           "bpp"

/**
 * MAFW_METADATA_KEY_EXIF_XML:
 *
 * Exif information in XML format. Its value is a string.
 */
#define MAFW_METADATA_KEY_EXIF_XML      "exif-xml"

/**
 * MAFW_METADATA_KEY_CHILDCOUNT:
 *
 * Describes the amount of child items the item has. Its value is an integer.
 */
#define MAFW_METADATA_KEY_CHILDCOUNT        "childcount"

/**
 * MAFW_METADATA_KEY_CHILDCOUNT_1:
 *
 * Describes the number of direct child items of item. Its value is an
 * integer.
 */
#define MAFW_METADATA_KEY_CHILDCOUNT_1		"childcount(1)"

/**
 * MAFW_METADATA_KEY_ICON_URI:
 *
 * An URI pointing to icon image. Its value is a string.
 */
#define MAFW_METADATA_KEY_ICON_URI      "icon-uri"

/**
 * MAFW_METADATA_KEY_ICON:
 *
 * Icon image. Its value is a byte array.
 */
#define MAFW_METADATA_KEY_ICON          "icon"

#endif /*MAFWMETADATA_H_*/
