/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_PLAYLIST_H
#define MAFW_PLAYLIST_H

#include <QObject>
#include <QStringList>
#include "MafwMediaInfo.h"

class QString;
class MafwContent;
class QFile;
class QAbstractItemModel;

/**
 * @brief The class representing the playlist in MAFW.
 *
 * Playlists can be shared and they can be modified and deleted by other
 * applications. Initially playlists are in memory only until an application
 * wants to share one.
 * Playlists can be shared using MafwShared::publishPlaylist from mafw-shared
 * library.
 * @see MafwInMemoryPlaylist for an actual implementation of this interface.
 */
class MafwPlaylist : public QObject
{
    Q_OBJECT

public:

    /** @enum MafwSortOrder The MafwPlaylist sorting order. */
    enum MafwSortOrder
    {
        AscendingOrder = Qt::AscendingOrder,/**< The items are sorted ascending. */
        DescendingOrder = Qt::DescendingOrder,/**< The items are sorted descending. */
        OriginalOrder,/**< The items are not sorted. */
        ShuffledOrder,/**< The items are shuffled. */
        AscendingNumOrder,/**< The items are sorted ascending and cell data is intepreted as integer. */
        DescendingNumOrder,/**< The items are sorted descending and cell data is intepreted as integer. */
        CustomOrder/**< The items are sorted custom order. */
    };

    /**
     * @enum MafwPlaylistUpdate Describes changes in playlist which
     * could be of interest to a renderer
     */
    enum MafwPlaylistUpdate
    {
        UpdateCurrentChanged,/**< The item at current index has changed, most likely removed. */
        UpdateCurrentMoved, /**< The current item's index has changed (shuffle/sort/move) */
        UpdateNext, /**< The item following the current item's index has changed.*/
        UpdateHasContent, /**< The playlist has new content, current index is uninitialized (-1).*/
        UpdateUnimportant /**< Update does not affect current index or next.*/
    };

public:

    /**
     * Constructs the playlist with given unique id. The id of MUST be unique,
     * otherwise publishing will fail. Maximum id length is 255 and it can
     * contain only the ASCII characters "[A-Z][a-z][0-9]_". Id can not start
     * with digit.
     * @param id The unique id of the playlist.
     * @param name The name of the playlist.
     * @see getNewPlaylistId()
     */
    MafwPlaylist( const QString& id, const QString& name = QString() ) :
                        m_name( name), m_id( id ), m_persistent(false)
    {}

    /**
     * The destructor
     */
    virtual ~MafwPlaylist(){}

    /**
     * Get the playlist name
     * @return Playlist name
     */
    virtual QString name() { return m_name; }

    /**
     * Set the playlist name
     * @param name The playlist name to be set.
     */
    void setName( const QString& name ) { m_name = name; }

    /**
     * Get the playlist id
     * @return the id of the playlist.
     */
    QString id() const { return m_id; }

    /**
     * Returns the model representing this playlist.
     * @return the Qt model object.
     */
    virtual QAbstractItemModel& model() = 0;

    /**
     * Sorts this playlist according to selected column number and the given order.
     * This function is asynchronous. Signal contentsChanged() will be emitted when the procedure is ready.
     * Passing MafwSortOrder::CustomOrder as sortOrder argument is invalid and will do nothing.
     *
     * To reset the order to the state how it was when the playlist was created, use
     * @code
     * sort( 0, MafwPlaylist::OriginalOrder );
     * @endcode
     * where column number is arbitrary (omitted by implementation).
     *
     * @see MafwSortOrder
     * @see MafwPlaylist::model()
     * @see QAbstractItemModel
     *
     * @param column the column number corresponding to underlying model
     * @param sortOrder the order
     */
    virtual void sort( int column, MafwSortOrder sortOrder = MafwPlaylist::AscendingOrder ) = 0;

    /**
     * Parses which column of the underlying model has the given name (such as "title")
     * and calls MafwPlaylist::sort(int, MafwSortOrder). getColumnNames() can be used
     * to query available names.
     *
     * @see MafwPlaylist::sort(int, MafwSortOrder)
     * @see MafwPlaylist::getColumnNames()
     */
    virtual void sort( QString columnName, MafwSortOrder sortOrder = MafwPlaylist::AscendingOrder ) = 0;

    /**
     * Convenience function to access field names in the underlying model.
     * @see MafwPlaylist::model
     * @return a list of column names
     */
    virtual QStringList getColumnNames() = 0;

    /**
     * Shuffle the playlist.
     */
    virtual void shuffle() = 0;

    /**
     * Unshuffle the playlist.
     */
    virtual void unShuffle() = 0;


    /**
     * Remove an item from certain index of the playlist.
     * @param index The index of item to be removed.
     * @return true if remove was successful, otherwise returns false
     */
    virtual bool removeItem( int index ) = 0;

    /**
     * Removes multiple items from the playlist.
     * @param firstIndex The index of the first item to be removed
     * @param lastIndex The index of the last item to be removed. If greater than the current
     * size of the playlist, all items from startIndex to the lastIndex
     * (inclusive) will be removed.
     * @return True if success, false otherwise
     */
    virtual bool removeItems(int firstIndex, int lastIndex ) = 0;

    /**
     * Move an item from a playlist index to another.
     * @param from The index to move item from.
     * @param to The index to move item to.
      * @return true if move was successful, otherwise returns false
     */
    virtual bool moveItem( int from, int to ) = 0;


    /**
     * Virtual function to clear the playlist.
     */
    virtual void clear()  = 0;

    /**
     * Get an item from a certain index of the playlist. If index is valid
     * the index of currently played item is updated. Usually this is called
     * by the renderer when it is playing the playlist.
     * @param index The index of the item to be fetched.
     * @return The item in the playlist index. Returns NULL, if no item
     * exists with an index.
     * Ownership transfers.
     * @deprecated
     */
    virtual MafwContent* getItem( int index ) = 0;

    /**
     * Get the size of the playlist.
     * @return the item count of the playlist
     */
    virtual int size() = 0;

    /**
     * Get the the current index on the playlist. The current index
     * is updated always when getItem(int index) is called with valid
     * playlist index.
     * @return the index of current item. Negative (-1) if no current.
     */
    virtual int getCurrentIndex() = 0;

    /**
     * Get the next index to play in playlist.
     * @param index The index of item that the next is asked from.
     * @return the index of the next item. Negative value (-1) if there is no next.
     */
    virtual int getNext( int index ) = 0;

    /**
     * Get the previous item of an item on the playlist.
     * @param index The index of item that the previous is asked from.
     * @return the previous item. Negative value (-1) if there is no previous.
     */
    virtual int getPrevious( int index ) = 0;

    /** @enum MafwRepeatMode The repeat mode. */
    enum MafwRepeatMode
    {
        RepeatOff = 0,/**< No repeat. */
        RepeatOne = 1,/**< Repeat single item. */
        RepeatAll = 2/**< Repeat all items. */
    };

    /**
     * Set repeat to the playlist.
     * @param mode The repeat mode.
     */
    virtual void setRepeat( MafwRepeatMode mode ) = 0;

    /**
     * Function to query if repeat is set to playlist.
     * @return Current repeat mode.
     */
    virtual MafwRepeatMode repeatMode() = 0;

    /**
     * Returns the current order in which this playlist is being presented.
     * @return current order
     */
    virtual MafwSortOrder getSortOrder() = 0;

    /**
     * Function to query if the playlist is shuffled.
     * This is equal to
     * @code
     * getSortOrder() == MafwSortOrder::ShuffledOrder
     * @endcode
     * @return true if the playlist is shuffled, otherwise returns false.
     */
    virtual bool isShuffled() = 0;

    /**
     * Virtual function to import an existing playlist from file.
     * @param file The playlist file.
     * @return true if import was successful, otherwise returns false.
     */
    virtual bool import( QFile& file ) = 0;

    /**
     * Get an item from a certain index of the playlist.
     * @credential TrackerReadAccess  Tracker read access needed if local content is used.
     * @param index The index of the item to be fetched.
     * @return The MafwMediaInfo of the playlist item in the index.
     */
    virtual MafwMediaInfo getItemMediaInfo(int index) { Q_UNUSED(index); return MafwMediaInfo(); }

    /**
     * Function to query if the playlist is persistent i.e. saved to the tracker.
     * @return true if the playlist is persistent, otherwise returns false.
     */
    virtual bool persistent(){ return m_persistent; }

    /**
     * Set the playlist persistent flag.
     * @param persistent true or false.
     */
    virtual void setPersistent(bool persistent){ m_persistent = persistent; }

    /**
     * Get the new unique string which can be used as playlist id.
     * @return new playlist id.
     */
    static QString getNewPlaylistId();

    /**
     * Convert a generated uuid (QUuid format) into a dbus friendly form
     * by removing '-', '{' and '}' characters and adding 'mafw'
     * "{xxxxxxxx-yyyy-xxxx-yyyy-xxxxxxxxxxxx}" -> "mafwxxxxxxxxyyyyxxxxyyyyxxxxxxxxxxxx"
     * @return converted playlist id.
     */
    static QString convertUuidtoPlaylistId(const QString& playlistUuid);

    /**
     * Reverts playlist id back to uuid standard format.
     * This function is intended to use for playlist ids generated using getNewPlaylistId()
     * or convertUuidtoPlaylistId(). For other ids functionality is unpredictable.
     * @see MafwPlaylistPersistencyHandler::savePlaylist
     * "mafwxxxxxxxxyyyyxxxxyyyyxxxxxxxxxxxx" -> "xxxxxxxx-yyyy-xxxx-yyyy-xxxxxxxxxxxx"
     * @return converted uuid
     */
    static QString convertPlaylistIdtoUuid(const QString& playlistId);

signals:
    /**
     * Signals that playlist content has been changed. Signal tells the index
     * of first item that has been modified and how many items from it has been
     * removed and how many items has been added.
     * @param from the index of the first element in the range of modified items.
     * @param remove number of items to remove, starting at from.
     * @param get number of items to to get, starting at from.
     */
    void contentsChanged( uint from, uint remove, uint get );

    /**
     * Signals generally changes to current or next item. The current item may
     * change i.e. it was deleted, or it can move to different location but
     * is still the same item i.e. shuffling/sort. In every case it is assumed that
     * the next item also changes. MafwPlaylistUpdate enum has type for case
     * in which only the next item changes. So only when the next item changes
     * the parameter newIndex can be ignored.
     * @param update Describes the type of change. @see MafwPlaylist::MafwPlaylistUpdate
     * for details
     * @param newIndex New index of the current item when it has changed/moved.
     */
    void playlistUpdated( MafwPlaylist::MafwPlaylistUpdate update, int newIndex);

    /**
     * Emitted when the playlist item has been moved.
     */
    void itemMoved();

    /**
     * Emitted when the playlist repeat mode has been changed.
     */
    void repeatModeChanged();

protected: //data
    /** The name of the playlist */
    QString m_name;

private: //data
    /**
     * The unique id of playlist. When playlist is shared
     * this property differs from zero.
     */
    QString m_id;

    /**
     * Is this playlist saved to tracker.
     */
    bool m_persistent;
};

#endif

