/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_PLUGIN_H
#define MAFW_PLUGIN_H

#include <QString>
#include <QtPlugin>

class MafwInternalRegistry;

/**
 * @brief The abstract base class for MAFW plugins.
 *
 * MafwPlugin is the abstract base class for all MAFW plugins.
 * MAFW plugins handle sources and renderers. Each source and renderer must
 * be handled by a plugin. One plugin may create several sources or renderers.
 *
 * For example "My remote service plugin" could handle a remote-server-source
 * and a remote-server-renderer. There could also be serveral remote servers
 * simultaneously, each handled by an instance of remote-server-source and all
 * the sources were created by the same plugin.
 *
 * Typically a source (or renderer) is shipped in a single library which contains
 * also the plugin class.
 *
 * MAFW plugins are loaded by MafwRegistry. The registry then calls the loaded plugin's
 * initialize() function, which instantiates any sources or renderers it will handle
 * and adds them to the registry.
 *
 * Below are some examples of loading and unloading of plugins. See MafwInternalRegistry
 * documentation for more details.
 * @code
 * registry->loadPlugins(); //attempts to load all available plugins from /usr/lib/qmafw
 * registry->loadPlugin("MyPluginName"); //attempts to load one plugin
 * registry->unloadPlugins(); //unloads all loaded plugins
 * registry->unloadPlugin("MyPluginName"); //unloads one plugin
 * @endcode
 *
 * Typically a source handles one type (or even just one instance) of
 * source or renderer. For example MafwGstRendererPlugin handles
 * 'MAFW GStreamer Renderer' which controls playback via GStreamer.
 *
 * MAFW plugins are Qt plugins and the rules for creating and loading Qt plugins also apply
 * to MAFW plugins. QPluginLoader class reference in Qt manual explains the process in more detail.
 * Qt manual's section "How to Create Qt Plugins" may also be useful to MAFW plugin developers.
 *
 * @see MafwSource
 * @see MafwRenderer
 * @see MafwRegistry
 * @see MafwInternalRegistry
 * @see QPluginLoader
 */
class MafwPlugin
{
public:
	virtual ~MafwPlugin() {}

	/**
	 * Initializes the plugin. Typically plugin instantiates source(s) and/or
     * renderer(s) and registers it using MafwInternalRegistry::addRenderer(MafwRenderer*)
     * or MafwInternalRegistry::addSource(MafwSource*).
     *
     * Initialize() is called by MafwInternalRegistry during plugin loading process.
     *
	 * @param registry Plugin registers sources and renderers to given registry.
	 * @see MafwInternalRegistry
	 */
	virtual void initialize(MafwInternalRegistry* registry) = 0;

	/**
     * Returns the human-readable name of this plugin.
     *
     * @note This is not the same as MafwSource::name() or MafwRenderer::name().
     *
	 * @return Name of this plugin
	 */
	virtual QString name() const = 0;
};

Q_DECLARE_INTERFACE(MafwPlugin, "MafwPlugin")

#endif
