/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_REGISTRY_H
#define MAFW_REGISTRY_H

#include <QObject>

class QPluginLoader;
class MafwRenderer;
class MafwSource;
class MafwPlaylist;
class MafwPlugin;
class MafwPluginControlInterface;



/*! \mainpage Media Application Framework
 *
 * \section intro_sec Introduction
 *
 * This is the introduction.
 *
 */

/**
 * @brief The MAFW registry class.
 * 
 * MafwRegistry is the starting point for an application using the framework.
 * It stores available renderers and sources, and provides signals for 
 * addition and removal events.
 *
 * Usual use case is that renderers and sources lies behind DBus. If that's 
 * the case, MafwShared::initTracking starts watching the session bus for 
 * appearing components. After the session bus tracking has been started 
 * signals rendererAdded, rendererRemoved, sourceAdded and sourceRemoved are 
 * signalled by registry.
 *
 * Sources and renderers are identified by an arbitrary long unique string,
 * called `Source ID' or `Renderer ID. The only restriction is that it MUST 
 * NOT contain the substring "::" (that is two colons).
 * 
 * After getting a registry instance, an application can load certain plugins 
 * or load all of them residing in /usr/lib/qmafw-plugin as in-process.
 * 
 * @code
 * MafwRegistry* registry = MafwRegistry::instance();
 *   
 * QList <MafwSource*> sources = registry->sources();
 * QList <MafwRenderer*> renderers = registry->renderers();
 *        
 * connect(registry, SIGNAL(rendererAdded(MafwRenderer*)),
 *         this, SLOT(rendererAdded(MafwRenderer*)));
 * connect(registry, SIGNAL(rendererRemoved(const MafwRenderer*)),
 *         this, SLOT(rendererRemoved(const MafwRenderer*)));
 * 
 * connect(registry, SIGNAL(sourceAdded(MafwSource*)),
 *         this, SLOT(sourceAdded(MafwSource*)));
 * connect(registry, SIGNAL(sourceRemoved(const MafwSource*)),
 *         this, SLOT(sourceRemoved(const MafwSource*)));
 * 
 * MafwShared* sessionBusTracker = new MafwShared();
 * sessionBusTracker->initTracking( registry );
 * 
 * @endcode
 * @see MafwShared::initTracking(  MafwRegistry* registry )
 */
class MafwRegistry : public QObject
{
    Q_OBJECT

    /************************************************************************
     * Initialization
     ************************************************************************/
private:
    Q_DISABLE_COPY(MafwRegistry)

protected:
    /**
     * The constructor.
     * Only MafwRegistry::instance() can create a new registry instance 
     * @param parent the parent object
     */
    MafwRegistry(QObject* parent);

public:

    /**
     * The singleton constructor.
     * @note Never delete this singleton instance manually.
     * @return the MafwRegistry singleton instance
     */
    static MafwRegistry* instance();

/************************************************************************
 * Sources
 ************************************************************************/

public:
    /** 
     * Get a source by its UUID 
     * @param uuid the uuid of the source to be found
     * @return the founded source object or NULL if source not found
     */
    virtual MafwSource* source(const QString& uuid) const = 0;
    
    /** 
     * List known sources 
     * @return list of sources in this registry
     */
    virtual const QList <MafwSource*> sources() const = 0;
    
Q_SIGNALS:
    /** 
     * Signal telling that a new source extension is available 
     * @deprecated use sourceAdded(const QString& uuid) instead
     * @param src the added source object
     */
    void sourceAdded(MafwSource* src);
        
    /** 
     * Signal telling that a new source extension is available 
     * @param uuid UUID of the added source
     * @param name name of the added source
     */
    void sourceAdded(const QString& uuid, const QString& name);
    
    /** 
     * Signal telling that a source is no longer available. The object may
     * already have been destroyed, so no method calls to it should be done.
     * @deprecated use sourceRemoved(const QString& uuid) instead
     * @param src the removed source
     */
    void sourceRemoved(const MafwSource* src);

    /** 
     * Signal telling that a source is no longer available. The object may
     * already have been destroyed, so no method calls to it should be done.
     * @param uuid the UUID of the removed source
     * @param name name of the removed source
     */
    void sourceRemoved(const QString& uuid, const QString& name);

 /************************************************************************
 * Renderers
 ************************************************************************/
    
public:
    /** 
     * Get renderer by its UUID 
     * @param uuid the uuid of the renderer to be found
     * @return the founded renderer object or NULL if renderer not found
     */
    virtual MafwRenderer* renderer(const QString& uuid) const = 0;

    /** 
     * List known renderers 
     * @return list of renderers in this registry
     */
    virtual const QList <MafwRenderer*> renderers() const = 0;

Q_SIGNALS:
    /** 
     * Signal telling that a new renderer extension is available 
     * @deprecated use rendererAdded(const QString& uuid) instead
     * @param rnd pointer to added renderer object
     */
    void rendererAdded(MafwRenderer* rnd);

    /** 
     * Signal telling that a new renderer extension is available 
     * @param uuid UUID of the added renderer
     * @param name name of the added renderer
     */
    void rendererAdded(const QString& uuid, const QString& name);
    
    /** 
     * Signal telling that a renderer is no longer available.
     * @deprecated use rendererRemoved(QString uuid) instead
     * @param rnd pointer to removed renderer
     */
    void rendererRemoved(const MafwRenderer* rnd);
    
    /** 
     * Signal telling that a renderer is no longer available.
     * @param uuid the UUID of the removed renderer
     * @param name name of the removed renderer
     */
    void rendererRemoved(const QString& uuid, const QString& name);
    
/************************************************************************
 * Playlists
 ************************************************************************/
    
public:

    /** 
     * Get a playlist by its id. Owned by registry.
     * @param id id of wanted playlist
     * @return pointer to found playlist or NULL if playlist not found
     */
    virtual MafwPlaylist* playlist(const QString& id) const = 0;

    /** 
     * List known playlists. Playlist items owned by registry.
     * @return list of all playlists in this registry
     */
    const virtual QList <MafwPlaylist*> playlists() const = 0;
    
    /** 
     * List persistent playlists loaded from tracker.
     * @return list of persistent playlists in this registry
     */
    const virtual QList <MafwPlaylist*> persistentPlaylists() const = 0;
    
Q_SIGNALS:
    /** 
     * Signal telling that a new playlist is available
     * @deprecated use playlistAdded(const QString& id) instead 
     * @param playlist Added playlist
     */
    void playlistAdded( MafwPlaylist* playlist );
    
    /** 
     * Signal telling that a new playlist is available 
     * @param id Id of the added playlist
     * @param name Name of the added playlist
     */
    void playlistAdded(const QString& id, const QString& name);
    
    /** 
     * Signal telling that a playlist is no longer available. The object may
     * already have been destroyed, so no method calls to it should be done.
     * @deprecated use playlistRemoved(const QString& id) instead
     * @param playlist Pointer to the emoved and deleted playlist
     */
    void playlistRemoved( const MafwPlaylist* playlist );

    /** 
     * Signal telling that a playlist is no longer available. The object may
     * already have been destroyed, so no method calls to it should be done.
     * @param id id of the playlist that has been removed
     * @param name name of the playlist that has been removed
     */
    void playlistRemoved( const QString& id, const QString& name );
    
/************************************************************************
 * Plugins
 ************************************************************************/
public:
	
    /** 
     * Load a plugin into the process from the given path.
     *
     * @param name the path where plugin will be loaded
     * @return TRUE when succeeded
     */
    virtual bool loadPlugin(const QString& name) = 0;

    /** 
     * Unload the given plugin 
     * 
     * @param name name of the plugin to be unloaded
     */
    virtual void unloadPlugin(const QString& name) = 0;

    /** 
     * Load all plugins into the process from MAFW_PLUGIN_PATH
     */
    virtual void loadPlugins() = 0;

    /** 
     * Unload all plugins 
     */
    virtual void unloadPlugins() = 0;

    /** 
     * Get a loaded plugin by its name 
     * @param name name of wanted plugin
     * @return pointer to founded plugin or NULL if plugin not founded
     */
    virtual MafwPlugin* plugin(const QString& name) const = 0;

    /** 
     * List plugins loaded in the registry
     * @return the list of plugins in this registry
     */
    virtual const QStringList loadedPlugins() const = 0;
    
    /** 
     * List known plugins 
     * @return the list of known plugins
     */
    virtual const QStringList availablePlugins() const = 0;

    /** 
     * Get a control interface of plugin by plugin name
     * @param pluginName name of wanted plugin
     * @return the founded control interface object or NULL if not found
     */
    virtual MafwPluginControlInterface* pluginControlInterface(const QString& pluginName) const = 0;
    
    /** 
     * List control interfaces for out-process plugins
     * @return list of control interfaces in this registry
     */
    virtual const QList <MafwPluginControlInterface*> pluginControlInterfaces() const = 0;
};

#endif
