/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_RENDERER_H
#define MAFW_RENDERER_H

#include <QObject>
#include <QString>
#include <QDebug>

class MafwCallbackHelper;
class MafwPlaylist;
class MafwContent;
class MafwMediaInfo;
class MafwError;
class QUrl;

/**
 * @brief The abstract base class for renderers. 
 * 
 * Provides various playback-related operations to control playback: 
 * play(), stop(), pause() and resume().
 */
class MafwRenderer : public QObject
{
    Q_OBJECT

    /** The Renderer ID */
    Q_PROPERTY(QString uuid READ uuid)

    /** the name of the plugin that has created this renderer */
    Q_PROPERTY(QString pluginName READ pluginName)

    /** the name of the renderer */
    Q_PROPERTY(QString name READ name)

    /** Plugins create renderer objects, so they can do whatever they will.
        Mostly this is here to let access to setUuid and setName. */
    friend class MafwPlugin;

private:
    Q_DISABLE_COPY(MafwRenderer)

public: //Properties

    /**
     * Policy property. The type of this property is boolean. Informing
     * if playback functionality has been overriden by policy,
     */
    static const QString MAFW_RENDERER_PROPERTY_POLICY_OVERRIDE;
    /**
     * Property for controllling playlist playback mode. The type of this property
     * is boolean. If set as true the renderer will automatically advance to the
     * next item in the playlist, if the next item exists. If set as false,
     * next() and then play() slots are required to be called to advance to the next item.
     */
    static const QString MAFW_RENDERER_PROPERTY_AUTO_ADVANCE;

public:
    /**
     * Constructor
     *
     * @note Name property should be initialized in the constructor of derived classes,
     * although it is not given in the parameter list.
     * @note uuid must not contain the substring "::".
     * @param uuid The uid of the renderer. The uuid is arbitrary long unique 
     * string. Its maximum length is 255 and it can contain only the ASCII 
     * characters "[A-Z][a-z][0-9]_". Id can not start with digit. 
     * @param pluginName The name of the renderer plugin.
     * @param parent The parent of this object
     */
    MafwRenderer(const QString& uuid, const QString& pluginName, QObject *parent = 0);

    /**
     * Destructor
     */
    virtual ~MafwRenderer();

    /**
     * Returns the renderer ID. Renderers are identified by an arbitrary long
     * unique string. The only restriction is that it MUST NOT contain the
     * substring "::" (that is two colons).
     * @return The universal unique identifier of the renderer.
     */
    virtual QString uuid() const;

    /**
     * Returns the name of the plugin that has created this renderer
     * @return the name of the plugin
     */
    virtual QString pluginName() const;

    /** Returns the name of the renderer
     * @return the name of the renderer
     */
    virtual QString name() const;

    /**
     * The seek mode for setPosition().
     */
    enum SeekMode {
        /**
         * Absolute means that the given seek time is counted as seconds
         * from the beginning of the current media. Only positive integers
         * are accepted.
         */
        SeekAbsolute,
        /**
         * Relative means that the given seek time is counted as seconds
         * from the current playback position. Use negative integers to seek
         * backwards and positive to seek forwards.
         */
        SeekRelative
        };


public Q_SLOTS:
    /**
     * Starts playback. Plays continously according to the playing order
     * specified for the playlist. If there is no assigned playlist, or the
     * renderer was already playing, does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     * @see assignPlaylist()
     */
    virtual bool play() = 0;

    /**
     * Starts playback with given URL.
     * @param url The URL to play
     * @param startPosition The position in seconds to start playback from, if seeking is possible for the media.
     * @note If parameter URL must be in encoded format to ensure correct behavior with URI delimiter characters(RFC3986).
     * For example when constructing an URL from string the string should be in encoded format (percent encoded)
     * and then create the URL with QUrl::fromEncoded(urlString); This makes sure the delimiter characters
     * are correctly encoded when taking the string representation with QUrl::toEncoded();
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool play(const QUrl& url, uint startPosition = 0) = 0;

    /**
     * Starts playback with given URL handles the URI as given content type.
     * This is used in case if URI can point to a playlist the contentType
     * should signify this. In this case renderer should play the playlist
     * as an single element, despite the amount of content in the playlist.
     * @see MafwRenderer::play(const QUrl) for URL format details.
     * @todo Currently use "audio/x-scpls" content type to tell renderer
     * to handle URI as an playlist.
     *
     * @param url The URL to play
     * @param contentType The type of the resource pointer by the URL.
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool play(const QUrl& url, const QString &contentType)
    {
        qWarning() << __PRETTY_FUNCTION__ << " not implemented.";
        Q_UNUSED(url);
        Q_UNUSED(contentType);
        return true;
    };

    /**
     * Stops playback. If playback is not ongoing does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool stop() = 0;

    /**
     * Pauses playback. If playback is not ongoing does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool pause() = 0;

    /**
     * Changes the state of the renderer from stopped state to paused
     * at the given position in the currently selected media (emitted via
     * mediaChanged() signal). PauseAt setting is 
     * reset when media is played. Application needs to call pauseAt again
     * for each time it wants to pause media at given position.
     * @note Works only in stopped state! Is ignored in any other states.
     * @param position The position in seconds where to seek in the stream.
     * If seeking is not possible the renderer goes to paused state at the
     * beginning of the stream and false is returned.
     * @return True if queuing succeeded, false if something went wrong.
     */
    virtual bool pauseAt(uint position) = 0;

    /**
     * Resumes playback. If playback is not in pause state does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool resume() = 0;

    /**
     * Skip to the next item in playlist. If there is no assigned playlist,
     * does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool next() = 0;

    /**
     * Skip to previous item in playlist. If there is no assigned playlist,
     * does nothing.
     *
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool previous() = 0;

    /**
     * Skip to given item in playlist. If there is no assigned playlist,
     * does nothing.
     *
     * @param index the index of item to go
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool gotoIndex(unsigned int index) = 0;

    /**
     * Set the current playback position (i.e. seek).
     *
     * @param seconds the seek time
     * @param mode the seek mode
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool setPosition(int seconds, SeekMode mode = SeekAbsolute) = 0;

public:

    /**
     * Gets the renderer status asynchronously. The results are delivered to the callback
     * slot. Possible formats of the callback slot are presented below:
     *
     * @code
     * public slot:
     *      void statusResults(QString playlistId, uint playbackIndex, int playbackState, QString objectId);
     * @endcode
     *
     * The method is invoked like:
     * @code
     * 		getStatus(this, SLOT(statusResults(QString, uint, int, QString)));
     * @endcode
     *
     * @param resultsReceiver The callback object
     * @param resultsMember The callback slot
     * @return true if queuing succeeded, false if something went wrong (e.g. slot couldn't be connected)
     */
    virtual bool getStatus(QObject *resultsReceiver, const char *resultsMember) = 0;

    /**
     * Assigns playlist to renderer.
     *
     * @param playlist to be assigned
     * @return true if queuing succeeded, false if something went wrong (e.g. DBus couldn't be connected)
     */
    virtual bool assignPlaylist(MafwPlaylist* playlist) = 0;

public:
    /**
     * Gets the current playback position asynchronously. The property is
     * delivered to the callback slot. Possible formats of the callback slot are presented below:
     *
     * @code
     * public slot:
     *      void positionResults(uint position);
     * @endcode
     *
     * The method is invoked like:
     * @code
     * 	    getPosition(this, SLOT(positionResults(uint)));
     * @endcode
     *
     *
     * @param resultsReceiver The callback object
     * @param resultsMember The callback slot
     * @return true if queuing succeeded, false if something went wrong (e.g. slot couldn't be connected)
     */
    virtual bool getPosition(QObject* resultsReceiver, const char* resultsMember) = 0;

public:
    /**
     * Set a property at renderer. Properties could be whatever class data 
     * members and MafwRenderer interface does not tell what properties 
     * renderer implementation has if any. 
     *
     * Implementation must emit mafwPropertyChanged(const QString& name, const QVariant& value),
     * when a property's value changes.
     *
     * @param name The name of the property
     * @param value The value of the property
     * @return True if success and property is predefined for this object, or property was added as dynamic property,
     *         false if setting failed(maybe wrong data type)
     * @see Q_PROPERTY()
     */
    virtual bool setMafwProperty(const QString& name, const QVariant& value);

    /**
     * Gets the MAFW property asynchronously.
     * The property is delivered to the callback slot. If a property with the
     * name does not exist, invalid QVariant is reported in the callback.
     * Format of the callback slot is presented below:
     *
     * @code
     * public slot:
     *      void propertyResult(QString& name, QVariant& value);
     * @endcode
     *
     * The method is invoked like:
     * @code
     * mafwProperty("prop1", this, SLOT(propertyResult(QString&, QVariant&)))
     * @endcode
     *
     * @param name The name of the property to fetch
     * @param receiver The callback object
     * @param member The callback slot
     */
    virtual bool mafwProperty(QString& name, QObject* receiver, const char* member);

public:
    /**
     * Returns all known info (metadata) about currently playing medium.
     * If the renderer is not in playing state, false is returned the callback will not be called.
     * The actual info may be accessed using MafwMediaInfo::metaData() or MafwMediaInfo::metaData(QString).
     *
     * If there is an error calling the callback slot or other renderer error occurs, the renderer emits error(const MafwError&).
     *
     * If metadataKey is given, but requested info is not known, the returned info
     * has empty value field.
     *
     * Callback slot form:
     * @code
     * public slots:
     *     void currentMediaInfoSlot( const MafwMediaInfo& info );
     *     void rendererError( const MafwError& error );
     * @endcode
     *
     * Connect to the error signal:
     * @code
     * connect(renderer, SIGNAL(error(MafwError)), this, SLOT(rendererError(MafwError)));
     * @endcode
     *
     * The method is invoked like this
     * @code
     * renderer->getCurrentMediaInfo(this, SLOT(currentMediaInfoSlot(const MafwMediaInfo&));
     * @endcode
     * or to get just the title:
     * @code
     * renderer->getCurrentMediaInfo(this, SLOT(currentMediaInfoSlot(const MafwMediaInfo&)), MAFW_MEDIA_KEY_TITLE);
     * @endcode
     *
     * @param receiver receiver the object whose method is called upon completion of the request
     * @param member The slot that is called upon completion of the request. The used callback slot
     *              has to be of form callbackname( const MafwContent& ).
     * @param metadataKey A single metadata key to request info only about one specific key.
     *                   If this is left empty, all known info about current media is returned.
     * @return true if queuing succeeded, false if something went wrong 
     * e.g. slot couldn't be connected or if the renderer is not in playing nor paused state.
     * @see MafwMediaInfo
     * @see MafwMetadata.h
     */
    //TODO Make this pure virtual after introduction phase.
    virtual bool getCurrentMediaInfo(QObject* receiver, const char* member, const QString& metadataKey=0)
    {
        Q_UNUSED(receiver);
        Q_UNUSED(member);
        Q_UNUSED(metadataKey);
        return false;
    }


public:
    /** The current playback state. */
    enum State
    {
        Stopped,/** playback state is stopped */
        Playing,/** playback state is playing */
        Paused,/** playback state is paused */
        Transitioning,/** playback state is transitioning */
        Invalid = 100 /** playback state is invalid */
    };


Q_SIGNALS:
    /**
     * Emitted when the renderer's playback state changes either thru
     * direct user action or during normal playback.
     * @param state the new state
     */
    void stateChanged(MafwRenderer::State state);

    /**
     * Playback has completed. An URI has been completed or the last item in a
     * playlist has been played and the playlist is not on any repeat mode.
     */
    void playbackCompleted();

    /**
     * Emitted when the renderer's current media changes either thru
     * direct user action or during normal playback.
     * @param index the index of changed media. If negative the current media
     * is not on the assigned playlist.
     * @param content the changed object
     */
    void mediaChanged(int index, const MafwContent& content);

    /**
     * Emitted when the renderer's current playlist changes either through
     * direct user action (or playlist destruction).
     * @param playlist The changed playlist, or zero '0' if no playlist assigned
     */
    void playlistChanged(MafwPlaylist* playlist);

    /**
     * Emitted when the metadata of the renderer's current media changes.
     * @param metadata	The metadata key that has changed
     * @param values	The new metadata values
     */
    void metadataChanged(const QString& metadata, const QList<QVariant>& values);

    /**
     * Emitted when the renderer is filling its playback buffer before or
     * during actual playback.
     * @param status the buffering status as a percent (0 - 100).
     */
    void bufferingInfo(float status);

    /**
     * Emitted when the renderer is scanning.
     * @param status The scanning status as a percent (0 - 100).
     */
    void scanningInfo(float status);

    /**
     * Emitted when the name of the renderer has changed.
     * param name The new name of the renderer 
     */
    void nameChanged(const QString& name);

    /**
     * Emitted when the property_changed signal is received from dbus.
     * @param name The name of the property
     * @param value The value to be set
     */
    void mafwPropertyChanged(const QString& name, const QVariant& value);

    /**
     * Emitted when renderer encounters an error situation.
     * @param error The MafwError describing the fail.
     */
    void error(const MafwError& error);

protected:

    /** the renderer ID */
    QString m_uuid;

    /** the plugin that has created this renderer */
    QString m_plugin;

    /** the name of the renderer */
    QString m_name;
};

#endif
