/*
 * This file is part of QMAFW
 *
 * Copyright (C) 2010 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_RENDERER_POLICY_H
#define MAFW_RENDERER_POLICY_H

#include <QObject>

/**
 * This class provides basis for renderer policy functionality. Inherit from
 * this class and implement the needed methods to create a renderer policy
 * object.
 */
class MafwRendererPolicy : public QObject
{
    Q_OBJECT

public:

    /** Policy group for the renderer */
    enum PolicyGroup
    {
        NoGroup = 0,    /** No group, i.e. uninitialized */
        MediaPlayer,    /** Media player, plays music and/or video */
        FmRadio         /** FM radio */
    };

    /** Resource enum to determine what resources renderer requires or can release */
    enum PolicyResourceFlag
    {
        Audio=0x1, /** Audio required */
        Video=0x2  /** Video resource */
    };

    /** Type for combination PolicyResourceFlag enums */
    Q_DECLARE_FLAGS(PolicyResourceFlags, PolicyResourceFlag)

    /**
     * Possible policy events
     */
    enum PolicyEvent
    {
        Granted,   /** access to resources granted, response to request() */
        Denied,    /** access to resources denied, response to request() */
        Stop,      /** policy asking to stop */
        Pause,     /** policy asking to pause */
    };

public:

    /**
     * Initializes the instance.
     * @note Must be called be the instance can be used.
     *
     * @param group The group renderer belongs to
     * @return True if successful, false otherwise.
     */
    virtual bool initialize( PolicyGroup group ) = 0;

    /**
     * Sets default set of resources that should be reserved
     * to enable playback. Some or all reserved resource can be later dropped
     * with release() function.
     * @param flags Flag containing the required resources enum values.
     */
    virtual void setDefaultResources(PolicyResourceFlags flags) = 0;

    /**
     * Implement this to request access to resources
     * Implementation must emit policyEvent as a response
     @param flags Flag containing the required resources enum values. Calling without flags request default resources.
     */
    virtual void request( PolicyResourceFlags flags=0 ) = 0;

    /**
     * Implement this to release resources
     * @param flags Resources to release. Calling without flags releases everything.
     */
    virtual void release( PolicyResourceFlags flags=0 ) = 0;

Q_SIGNALS:

    /**
     * Emit this signal as a response to a request() or to indicate other policy
     * events.
     * @param event The policy event that occurred.
     */
    void policyEvent(MafwRendererPolicy::PolicyEvent event);

};

Q_DECLARE_OPERATORS_FOR_FLAGS(MafwRendererPolicy::PolicyResourceFlags);

#endif // MAFW_RENDERER_POLICY_H
