/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFW_SOURCE_H
#define MAFW_SOURCE_H

#include <QObject>
#include <QVariant>
#include <QStringList>

class MafwContent;
class MafwPlugin;

/**
 * @brief The abstract base class which individual sources should inherit from. 
 * 
 * Sources represent repositories of multimedia contents, which
 * can either reside in the local file system or on a remote server. 
 * 
 * Object IDs:
 *
 * Individual content items are identified by their `Object ID'. Sources
 * produce and renderers consume Object IDs, which are only meaningful for the
 * providing sources themselves. An object ID is a string:
 * @code Source ID::Item ID @endcode consisting of the Source ID of the 
 * originating source and a unique (within the individual source's scope) 
 * Item ID, separated by two colons. Each object ID should be unique,
 * persistent and portable, if possible. Applications may assume that an 
 * object ID corresponds to a single content item, and this should be 
 * maintained across shutdowns, reboots and reinstalls.
 *
 * MafwSource's can be acquired from a MafwRegistry. The main
 * operations of a source are browsing or querying for content items.
 * @warning Be carefull when processing result data, it is not guaranteed to be valid.
 * 
 * @par Browsing via LibQtTracker
 * Browsing can be done also via the LibQtTracker library. By this the ontology
 * class hierarchy of LibQtTracker can utilized to some extend. MafwSources
 * support mainly media specific ontologies.
 * The next example shows how the contents of a sources root can be browsed.
 * @code
 * QSharedPointer<RDFService> service = BackEnds::Mafw::getRDFService(*m_sources.at(selectedIndex));
 * LiveNode root = BackEnds::Mafw::rootNode(service);
 * Live<nie::InformationElement> ie = root;
 * //declared: LiveNodes m_browseLiveNodes;
 * m_browseLiveNodes = ie->getHasLogicalParts();
 * @endcode
 * The next code example shows how a browsing can be done to an selected item
 * in a Qt view.
 * @code
 * //getSelectedIndex returns the currently selected item in a view
 * QModelIndex index = getSelectedIndex();
 * QVariant data = index.data(SopranoLive::LiveNodeModel::LiveNodeRole);
 * LiveNode node = data.value<LiveNode>();
 * Live<nie::InformationElement> live = node;
 * //declared: LiveNodes m_browseTempResults;
 * m_browseTempResuts = live->getHasLogicalParts();
 * @endcode
 * @warning Be careful when making assignments to LiveNodes variables. The
 * LiveNodes works as a sharedpointer, so if you assign a new LiveNodes object
 * to a variable the old LiveNodes object may be deleted if it was the last
 * references to it. This means that also the Qt model it provided is deleted.
 * This may cause weird segmentation faults.
 * @par Limitations
 * @todo
 * Unfortunately the current implementation level of browsing via LibQtTracker
 * is limited to most basic browsing. The browsing returns a table with an Mafw
 * object Id in the first column and the title of the item in the second column.
 * The model in the LiveNodes::model() can be placed in a MafwCFPlaylist and
 * if it contains playable items, these items can be played. Improvements
 * are underway...
 */
class MafwSource : public QObject
{
	Q_OBJECT

	/** The name of the source **/
	Q_PROPERTY(QString name READ name)
	
    friend class MafwPlugin;

private:
	Q_DISABLE_COPY(MafwSource)

public:
    /**
     * Parses the ID of the source from the given objectId
     * @param objectId The objectId to parse
     * @return The source part of the objectId
     */
    static QString parseSourceId(const QString objectId);

public:
    /**
     * Constructor
     * 
     * @note Name property should be initialized in the constructor of derived classes,
     * although it is not given in the parameter list.
     * 
     * @param parent the parent object
     * @param uuid universal unique identifier of this source. The uuid is 
     * arbitrary long unique string. Its Maximum length is 255 and it can 
     * contain only the ASCII characters "[A-Z][a-z][0-9]_". Id can not start 
     * with digit.
     * @param plugin the name of the plugin
     */
    MafwSource(QObject* parent, const QString& uuid, const QString& plugin);
	
    /**
     * Destructor
     */
    virtual ~MafwSource();

    /**
     * Returns the Source ID. Sources are identified by an arbitrary long 
     * unique string. The only restriction is that it MUST NOT contain the 
     * substring "::" (that is two colons).
     */
    virtual QString uuid() const;
    
    /**
     * Returns the name of the plugin that has created this source
     */
    virtual QString pluginName() const;

    /**
     * Returns the name of the source.
     * @note The name doesn't have to be unique, use uuid() if you want to identify the source.
     */
    virtual QString name() const;

    /**
     * Browse a source's content, using RDF predicates. This method intended
     * to be used by libqttracker to implement browsing functionality.
     * This default implementation does nothing. At tihe time of writing only
     * UPnP source implements this.
     * @param objectId The id of the object to browse, for a container to get
     * content (children).
     * @param predicate RDF predicate to get the value for. In case of containers, use
     * "nie::hasLogicalPart" predicate to get children of the container.
     * @param additionalPredicates Additional "metadata" predicates to be included in the
     * browse e.g. "nie::Title".
     * @param skipCount The amount of items to skip from the beginning.
     * @param itemCount The maximum amount of items to return.
     * @param receiver The object to receive the result/error signals
     * @param resultSlot The slot to receive the results. The slot signature:
     * @code
     * void resultSlot(uint queryId, QString subjectId,
     *      QMap<QString,QVariant> meta, uint index, int remainingCount)
     * @endcode
     * @param errorSlot The slot to receive possible error codes. The slot signature:
     * @code
     * void errorSlot( uint browseId, const int& error );
     * @endcode
     * The error is a MafwError::Code.
     * @return Id for the browse request. Zero if browse cannot executed at all.
     * 
     * Browse properties of given resource node. If given predicate is detected to point to resource node
     * then fetch also additionalPredicates from that node.
     * 
     * Example invocation:
     * @code
     * source->browse( node.iri().toString(), rdfs::type::iri().toString(), QStringList(), 0, 5, this,
                SLOT( browseResultSlot(uint, QString, QMap<QString,QVariant>, uint, int) ),
                SLOT( browseErrorSlot(uint, int, QString) ) );
     * @endcode
     * 
     * Callback slot signatures:
     * @code
     * public slot:
     *    void handleBrowseResult(uint browseId, QString objectId, QMap<QString,QVariant> metamap, uint index, int remainingCount);
     * @endcode
     *
     * @code
     * public slot:
     *    void handleBrowseError(uint queryId, int errorCode, QString errorMessage);
     * @endcode
     * 
     */
    virtual uint browse( const QString objectId, const QString predicate, const QStringList additionalPredicates, 
                        uint skipCount, uint itemCount, 
                        QObject* receiver, const char* resultSlot,
                        const char* errorSlot ) = 0;  
    
    /**
     * Cancels ongoing browse. Browsing is cancelled immediately, but 
     * available result will still come. The error slot given by browse call
     * is called when cancel is made.
     * @param browseId the browse request id to be cancelled
     * @todo make this pure virtual when this is implemented in sources.
     */
    virtual void cancelBrowse( uint browseId ) { Q_UNUSED(browseId) }
    
    /**
     * Query method performs a query on a source. The results of the query are 
     * delivered to the slot of the given receiver specified by the member argument.
     * Format of the callback slot is presented below:
     * 
     * @code
     * public slot:
     *    void handleQueryResult(uint queryId, const QStringList& resultRow, uint index, int remainingCount);
     * @endcode
     * When the query is finished, the error slot is called. Also if an error happens
     * before the query is succesfully finished, the error slot is called.
     * Format of the error slot is presented below:
     *
     * @code
     * public slot:
     *    void handleQueryError(uint queryId, int errorCode, QString errorMessage);
     * @endcode
     * 
     * Example invocation:
     * @code
     * source->query( sparql, this,
                SLOT( queryResultSlot(uint, const QStringList&, uint, int);
                SLOT( queryErrorSlot(uint, int, QString) ) );
     * @endcode
     * 
     * @param query The well-formatted SPARQL query to be handled by the source.
     * @param receiver The QObject whose result and/or error slots are called.
     * @param resultSlot The slot used to receive result rows.
     * @param errorSlot The slot is invoked at the end of query or when an error occurs.
     * @return The id of of the query. 
     */
    virtual uint query(QString query, QObject* receiver, const char* resultSlot,
                       const char* errorSlot) = 0;
  
    /**
     * Cancels ongoing query. Query is cancelled immediately, but
     * available result will still come. The error slot given by query call
     * is called when cancel is made.
     * @param queryId the query request id to be cancelled
     * @todo make this pure virtual when this is implemented in sources.
     */
    virtual void cancelQuery( uint queryId ) { Q_UNUSED(queryId) }

    /**
     * Set a property at source. Properties can be whatever class data 
     * members. MafwRenderer interface does not tell what properties a 
     * renderer implementation has, if any. 
     * 
     * @param name The name of the property
     * @param value The value of the property
     * @return true if success and property is predefined for this object, or property was added as dynamic property,
     *         false if setting failed(maybe wrong data type or property is read only)
     */ 
    virtual bool setMafwProperty(const QString& name, const QVariant& value) = 0;
    
    /**
     * Gets the MAFW property asynchronously. The property is delivered to the callback 
     * slot. Format of the callback slot is presented below:
     *
     * @code
     * public slot:
     *      void propertyChanged(QString& name, QVariant& value);
     * @endcode
     *
     * The method is invoked like:
     * @code
     * if (mafwProperty("prop1", this, SLOT(propertyChanged(QString&, QVariant&)) ))
     * @endcode
     * 
     * @param name The name of the property to fetch
     * @param receiver The callback object
     * @param member The callback slot
     * 
     * @return true if success
     */
    virtual bool mafwProperty(QString& name, QObject* receiver, const char* member) = 0;
    
protected:
	
	/**
	 * Sets a new name for this source. Emits nameChanged(Qstring&). There is 
	 * no need that name must be unique.
	 */
	void setName(QString name);
	
	/**
     * Returns the name of the plugin that has created this source
     */
	//Currently this should be set only in MafwSource constructor,
	//thus no setter is needed - we have access to the private variable m_plugin.
	//void setPluginName(QString pluginName);
	
	/**
     * Sets the Source ID. Sources are identified by an arbitrary long 
     * unique string. The only restriction is that it MUST NOT contain the 
     * substring "::" (that is two colons).
     */
	void setUuid(QString uuid);
	
private:
	
    /** the name of the plugin that has created this source */
    QString m_plugin;
    
    /** the Source ID */
    QString m_uuid;
    
    /** the source name */
    QString m_name;

signals:
	/** 
	 * Signal telling that the contents of a container object have changed. 
	 * @param object the modified container
	 */
	void containerChanged(const MafwContent& object);
	
	/** 
	 * Signal telling that the metadata of an object has changed.
	 * @param object the modified container
	 */
	void metadataChanged(const MafwContent& object);
	
	/**
	 * Signal telling that the name of the source has changed.
	 * @param name of the source
	 */
	void nameChanged(QString& name);
	
	/**
	 * Signal telling that browse request has been completed
	 */
	void browseCompleted();

    /**
     * Emitted when the property has changed.
     * @param name The name of the property
     * @param value new value
     */
    void mafwPropertyChanged(const QString& name, const QVariant& value);
};

#endif
