/* 
 * This file is part of QMAFW 
 *
 * Copyright (C) 2009 Nokia Corporation and/or its subsidiary(-ies). All rights
 * reserved.
 *
 * Contact: Visa Smolander <visa.smolander@nokia.com>
 *
 * This software, including documentation, is protected by copyright controlled
 * by Nokia Corporation. All rights are reserved. Copying, including
 * reproducing, storing, adapting or translating, any or all of this material
 * requires the prior written consent of Nokia Corporation. This material also
 * contains confidential information which may not be disclosed to others
 * without the prior written consent of Nokia.
 *
 */

#ifndef MAFWSPARQLPARSER_
#define MAFWSPARQLPARSER_

#include <QObject>
#include <QMap>
#include <QStringList>

/**
 * @brief Node represents either subject or predicate of SPARQL statement. 
 *
 * Node is typically created by parser.
 */
class MafwNode : public QObject
{   
public:
    /**
     * Node types. Users can define own types starting from NodeUserType.
     */
    enum MafwNodeType { NodeUndefined=0, NodeVariable, NodeBlank, NodeResource, NodeLiteral, NodeUserType=100 };

    /**
     * Initializes Node from string.
     * @param str Node string from SPARQL statement. E.g. "?title" or "<http://www.tracker-project.org/temp/nmm#MusicPiece>".
     * @return True on succee
     */
    bool init( QString str );
    
    /**
     * Get the type of node.
     * @return Node's type
     */
    MafwNodeType nodeType() const;
    
    /**
     * Get the text of node.
     * @return Text in node. If node is resource the '<' '>' signs are included. If it is literal then quote marks are included.
     */
    QString text() const;
    
    /**
     * Set the type of the node.
     * @param type New type for node
     */
    void setNodeType(MafwNodeType type);
    
    /**
     * Set the node text.
     * @param text New text for node.
     */
    void setText(QString text);
    
protected:
    /** Type of this node */
    MafwNodeType m_type;
    /** Text in this node */
    QString m_text;
};

/**
 * @brief Class representing SPARQL statement. 
 *
 * Statement consists of subject, predicate and object. Typically created by parser.
 */
class MafwStatement : public QObject
{
public:
    /**
     * Constructor.
     */
    MafwStatement();
    
    /**
     * Get the subject of statement.
     * @return Subject of this Statement
     */
    MafwNode& subject();
    
    /**
     * Get the predicate of statement.
     * @return Predicate of this Statement
     */
    QString predicate() const;
    
    /**
     * Get the object of statement.
     * @return Object of this Statement
     */
    MafwNode& object();
    
    /**
     * Set the predicate of statement.
     * @param predicate New preciate to set
     */
    void setPredicate(QString predicate);
    
protected:
    /** The subject of this statement */
    MafwNode m_subject;
    /** The predicate of this statement */
    QString m_predicate;
    /** The object of this statement */
    MafwNode m_object;
};

/**
 * @brief Class representing FILTER for certain variable at SPARQL query. 
 * 
 * Typically created by parser.
 */
class MafwFilter : public QObject
{
public:
    /**
     * Filter types. Users can define own types starting from FILTER_USER_TYPE.
     */
    enum MafwFilterType { FilterUndefined=0, FilterRegex, FilterEquals, FilterLess, FilterGreater, FilterUserType };

    /**
     * Create new Filter from given type and constrait.
     * @param constraint The filter's constraint text
     * @param type The filter's type
     */ 
    MafwFilter( QString constraint, MafwFilterType type );
    
    /**
     * Get the type of the filter.
     * @return Filter's type
     */
    MafwFilterType filterType() const;
    
    /**
     * Get the constraint of the filter.
     * @return Filter's constraint text
     */
    QString constraint() const;
    
protected:
    /**
     * Type of this filter.
     */
    MafwFilterType m_type;
    /**
     * Constraint of this filter.
     */
    QString m_constraint;
};

/**
 * @brief Class representing subquery of SPARQL query. 
 *
 * Statements can be arranged to subqueries using MafwSparqlParser::subqueryAnalysis().
 */
class MafwSubquery : public QObject
{
public:
    /**
     * Create new Subquery from given variables and statements.
     * @param variables
     * @param statements
     */
    MafwSubquery( QList<QString> variables, QList<MafwStatement*> statements );
    ~MafwSubquery();
    
    /**
     * Get the variables of the subquery.
     * @return List of variables (names) present in this subquery
     */
    QList<QString>& variables();
    
    /**
     * Get the statements of the subquery.
     * @return List of Statements belonging to this subquery. Ownership is not transfered.
     */
    QList<MafwStatement*> statements() const;
    
protected:
    /**
     * List of variable names
     */
    QList<QString> m_variables;
    
    /**
     * Statements are owned by Subquery.
     */ 
    QList<MafwStatement*> m_statements;
};

/**
 * @brief Simple SPARQL parser utility class.
 * 
 * Parse using parse() and then access parsed query model with statements(), filters() and orderBy().
 * Optionally use subqueryAnalysis() that will split parsed statements to subqueries and tries to arrange them to sensible execution order.
 * Accessed with subqueries().
 * 
 * @code
 * parse("SELECT ?x ?title WHERE { ?x <http://www.w3.org/1999/02/22-rdf-syntax-ns#type> ?poo .
 *           FILTER(?poo = <http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Media> ) . 
 *          ?x <http://www.semanticdesktop.org/ontologies/2007/01/19/nie#title> ?title .
 *          }");
 * @endcode
 * After parsing this there is
 * - two variables (?x and ?title) ( available with variables() )
 * - two statements (available with statements() ) (both have NodeVariable type subject and object Nodes)
 * - one FilterEquals type MafwFilter with constraint 
 *   <http://www.semanticdesktop.org/ontologies/2007/03/22/nfo#Media> 
 *   for ?poo variable ( available with filters() )
 */
class MafwSparqlParser : public QObject
{
    friend class Ut_MafwSparqlParser;
public:
    /*
     * Error codes returned by parse().
     */

    /**
     * Syntax error
     */
    static const int SYNTAX_ERROR = 1;

    /**
     * Other error
     */
    static const int OTHER_ERROR = 2;

    /**
     * Extended classes can define own error codes starting from this value.
     */
    static const int EXTENSION_ERROR_BASE = 100;    
    
public:
    MafwSparqlParser();
    ~MafwSparqlParser();
    
    /**
     * Parse given sparql to statements and filters.
     * @param sparql    Sparql string
     * @return Error code (zero on succee)
     */
    int parse(QString sparql);
       
    /**
     * Split query to subqueries. After calling this subqueries are accessed using subqueries() and statements() will return empty list.
     */
    void subqueryAnalysis();
    
    /**
     * Get the variables of the parsed SPARQL statements.
     * @return List of variable names.
     */
    QList<QString>& variables();
    
    /**
     * Get the statements of the parsed SPARQL query.
     * @return Statements. Ownership is not transfered.
     */
    QList<MafwStatement*>& statements();
    
    /**
     * Get the filters of the parsed SPARQL query.
     * @return Variables to Filters mapping. Ownership is not transfered.
     */
    QMap<QString,MafwFilter*>& filters();
    
    /**
     * Get the subqueries of the parsed SPARQL query.
     * @return subqueries in order. Ownership is not transfered.
     */
    QList<MafwSubquery*>& subqueries();
    
    /**
     * Get the variable by which results should be ordered.
     * @return Variable by which results should be ordered.
     */
    QString orderBy() const;

    /**
     * Returns Sparql LIMIT clause value.
     * @return Negative if not limit set, otherwise the value.
     */
    int limit() const;

    /**
     * Returns Sparql OFFSET clause value.
     * @return Negative if not offset set, otherwise the value.
     */
    int offset() const;
    
    /**
     * Clear all information.
     */
    void reset();
    
protected:
    /**
     * Return how many variables of given subquery (index) are not introduced in previous subqueries.
     * @param sub Index of subquery
     * @return Number of unknown variables
     */    
    uint notResolvedVarsAtSub(uint sub);
    
    /**
     * Split sparql string to token list.
     */
    QStringList tokenizeSparql( QString sparql );
    
    /**
     * Ensure that there is space around given character position. Used in preprocessing sparql string before tokenization.
     * @param string String to modify
     * @param pos Position to ensure
     */
    void ensureSpace(QString& string, int pos);
    
    /**
     * Parse one SPARQL triplet to Statement object from given token list. Triplet is removed from list.
     */
    int parseStatement( QStringList& tokens );
    
    /**
     * Parse FILTER clause to Filter object from given token list. Related tokens are removed from list.
     */
    int parseFilter( QStringList& tokens );
    
    /**
     * Removed OPTIONAL block's start and end marks from token list.
     */
    int removeOptional( QStringList& tokens );

    /**
     * Parse ORDER BY clause from given token list. Related tokens are removed from list.
     */
    int parseOrderBy( QStringList& tokens );

    /**
     * Parse OFFSET clause from given token list. Related tokens are removed from list.
     */
    int parseOffset(QStringList &tokens);

    /**
     * Parse LIMIT clause from given token list. Related tokens are removed from list.
     */
    int parseLimit( QStringList &tokens);
    
    /**
     * Find statements with given subject and create new subquery of them. Used by subqueryAnalysis().
     */
    void deriveSubquery( QString subject );
        
    /**
     * Remove first and last character. Used to remove " ", < > or () marks.
     */
    static QString center( QString str );

    /**
     * Variables in query
     */
    QList<QString> m_variables;
    
    /**
     * After parsing statements are stored here. subqueryAnalysis() (if used) moves them to Subqueries in m_subqueries.
     */
    QList<MafwStatement*> m_statements;
    
    /**
     * Variables to Filters mapping.
     */
    QMap<QString,MafwFilter*> m_filters;
    
    /**
     * Subqueries in order.
     */
    QList<MafwSubquery*> m_subqueries;
    
    /**
     * Variable by which results should be ordered.
     */
    QString m_orderBy;

    /**
     * Variable from which the results should be returned
     */
    int m_offset;

    /**
     * Variable which limits the results amount
     */
    int m_limit;
};

#endif /*MAFWSPARQLPARSER_*/
